package kudos.ide.action.key;

import java.awt.event.*;
import javax.swing.text.*;

import kudos.ide.buffer.*;
import kudos.ide.form.*;
import kudos.ide.form.textarea.*;
import kudos.ide.tools.*;

public class Tab extends EditAction
{
	private int selectStartLength;
	private int selectEndLength;

	public void actionPerformed(ActionEvent evt)
	{
		MainView mainView = getMainView(evt);
		Buffer buffer = mainView.getBuffer();
		EditTextArea editTextArea = mainView.getEditTextArea();

		if(!editTextArea.isEditable())
		{
			mainView.getToolkit().beep();
			return;
		}
		
		int selStart = editTextArea.getSelectionStart();
		int selEnd = editTextArea.getSelectionEnd();
		
		if(selStart == selEnd
			&& buffer.indentLine(editTextArea,editTextArea
			.getSelectionStartLine(),false))
		{
			return;
		}
		if(selStart != selEnd)
		{
			selectTab(evt);
			return;
		}
		editTextArea.setSelectedText("\t");
	}

	public void selectTab(ActionEvent evt)
	{
		MainView mainView = getMainView(evt);
		Buffer buffer = mainView.getBuffer();
		buffer.beginCompoundEdit();
		try
		{
			EditTextArea editTextArea = mainView.getEditTextArea();
			if(!editTextArea.isEditable())
			{
				mainView.getToolkit().beep();
				return;
			}
			int tabSize = buffer.getTabSize();
			Element map = buffer.getDefaultRootElement();
			int start = editTextArea.getSelectionStartLine();
			int end = editTextArea.getSelectionEndLine();
			for(int i = start; i <= end; i++)
			{
				Element lineElement = map.getElement(i);
				int lineStart = lineElement.getStartOffset();
				String line = buffer.getText(lineStart,
					lineElement.getEndOffset() - lineStart - 1);
				int whiteSpace = Util.getLeadingWhiteSpace(line);
				int whiteSpaceWidth = Util.getLeadingWhiteSpaceWidth(
					line,tabSize) + tabSize;
				buffer.remove(lineStart,whiteSpace);
				buffer.insertString(lineStart,Util
					.createWhiteSpace(whiteSpaceWidth,tabSize),null);
			}

			selectStartLength = 0;
			if(start != 0)
			{
				selectStartLength = 0;
				for(int i = 0; i < start; i++)
				{
					selectStartLength += editTextArea.getLineLength(i) + 1;
				}
			}
			selectEndLength = -1;
			for(int i = 0; i <= end; i++)
			{
				selectEndLength += editTextArea.getLineLength(i) + 1;
			}

			editTextArea.select(selectStartLength, selectEndLength);
		}
		catch(BadLocationException bl){}
		finally
		{
			buffer.endCompoundEdit();
		}
	}
}