/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * ClassInfo.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.ide.form;

import java.lang.reflect.*;

/**
 * Utility function that makes up the information of the given class.
 *
 * @author Cheoljoo Jeong
 */
public class ClassInfo {
  
  /**
   * Returns the information on the given class.
   */
  public static String getInfo(Class theClass) {
    if (theClass == null) {
      return "";
    }
    String text = theClass.getName() + "\n";

    // superclass
    text = text + "  extends " + 
      theClass.getSuperclass().getName() + " {\n";

    // fields
    Field[] fields = theClass.getDeclaredFields();
    for (int i = 0; i < fields.length; i++) {
      Class fieldty = fields[i].getType();
      text = text + "  " + getTypeName(fieldty) + " " +
	fields[i].getName() + ";\n";
    }
    text += "\n";

    // methods
    Method[] methods = theClass.getDeclaredMethods();
    for (int i = 0; i < methods.length; i++) {
      Class returnty = methods[i].getReturnType();
      text = text + "  " + getTypeName(returnty) + " " + methods[i].getName();
      Class[] paramtys = methods[i].getParameterTypes();
      if (paramtys.length > 0) {
	text += "(";
	for (int j = 0; j < paramtys.length; j++) {
	  if (j == 0) {
	    text = text + getTypeName(paramtys[j]) + ",";
	  }
	  if (j < paramtys.length - 1) {
	    text = text + "    " + getTypeName(paramtys[j]) + ",";
	  }
	  else {
	    text = text + "    " + getTypeName(paramtys[j]) + ");\n";
	  }
	}
      }
      else {
	text += "();\n";
      }
    }
    text += "}\n";

    return text;
  }

  /**
   * Returns the name of the class (including primitive types).
   */
  public static String getTypeName(Class classty) {
    return classty.getName();
  }

}

