package kudos.ide.form;

import java.io.*;
import java.awt.*;
import javax.swing.*;

import kudos.ide.io.*;
import kudos.ide.buffer.*;
import kudos.ide.tools.*;

public class EditFileDialog extends VirtualFileSystem
{
  public Buffer showOpenDialog(MainView mainView, Buffer buffer)
  {
    if(buffer.isDirty())
    {
      int result = JOptionPane.showConfirmDialog(mainView,
						 ("Save changes to " + buffer.toString() + "?"),
						 "File Not Saved", JOptionPane.YES_NO_CANCEL_OPTION,
						 JOptionPane.WARNING_MESSAGE);
      if(result == JOptionPane.YES_OPTION)
	buffer.save(mainView,null);
      if(result == JOptionPane.CANCEL_OPTION)
	return buffer;
    }
    int retval;
    JFileChooser chooser = new JFileChooser(new File("."));
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

    TFileFilter scmFilter = new TFileFilter("scm", "SchemeJ Files");
    chooser.addChoosableFileFilter(scmFilter);

    retval = chooser.showOpenDialog(mainView);
    chooser.setApproveButtonText("Open File...");

    if (retval == JFileChooser.APPROVE_OPTION)
    {
      File file = chooser.getSelectedFile();
      if(file.getPath() != null && file != null)
	return MainControl.openFile(mainView,null,file.getPath(),false,false,null);
    }
    return null;
  }

  public String showSaveDialog(MainView mainView, Buffer buffer)
  {
    int retval;
    JFileChooser chooser = new JFileChooser(new File("."));
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

    TFileFilter scmFilter = new TFileFilter("scm", "SchemeJ Files");
    chooser.addChoosableFileFilter(scmFilter);

    retval = chooser.showSaveDialog(mainView);
    chooser.setApproveButtonText("Save File...");

    if (retval == JFileChooser.APPROVE_OPTION)
    {
      File file = chooser.getSelectedFile();
      if(file.getPath() != null)
	return file.getPath();
    }
    return null;
  }
	
  public Buffer showInsertDialog(MainView mainView, Buffer buffer)
  {
    int retval;
    JFileChooser chooser = new JFileChooser(new File("."));
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

    TFileFilter scmFilter = new TFileFilter("scm", "SchemeJ Files");
    chooser.addChoosableFileFilter(scmFilter);

    retval = chooser.showDialog(mainView, "Insert File...");

    if (retval == JFileChooser.APPROVE_OPTION)
    {
      File file = chooser.getSelectedFile();
      if(file.getPath() != null && file != null)
	return MainControl.insertFile(mainView,file.getPath());
    }
    return null;
  }

  public InputStream _createInputStream(MainView mainView, Buffer buffer,
					String path, boolean ignoreErrors) throws IOException
  {
    try
    {
      return new FileInputStream(path);
    }
    catch(IOException io)
    {
      if(ignoreErrors)
	return null;
      else
	throw io;
    }
  }

  public OutputStream _createOutputStream(MainView mainView, Buffer buffer, String path)
    throws IOException
  {
    return new FileOutputStream(path);
  }

  public boolean load(MainView mainView, Buffer buffer, String path)
  {
    File file = new File(path);

    return super.load(mainView,buffer,path);
  }

  public boolean save(MainView mainView, Buffer buffer, String path)
  {
    File file = new File(path);

    return super.save(mainView,buffer,path);
  }
}
