/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * ListReader.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.ide.form;



/**
 * ListReader reads and returns a s-expression (either an atom or a list)
 * from an array of characters.
 */
public class ListReader {
  public static final String EOS = "_EOS_";
  private char[] source;
  private int sptr;
  private int ssize;
  
  /**
   * constructor
   */
  public ListReader(String text) {
    source = text.toCharArray();
    sptr = 0;
    ssize = text.length();
  }

  /**
   * Returns a s-expression from the source. (The name of this
   * method is somewhat misleading.)
   * An s-expr is either a list or an atom.
   * If `end-of-source' is encountered, static constant
   * EOS is returned.
   */
  public String readList() {
    StringBuffer buffer = new StringBuffer();
    int leftpar = 0;
    int rightpar = 0;
    char c = 'c';
    
    for (;;) {
      // if source not exhausted
      if (sptr < ssize) {
	// read a single char from a char list
	c = source[sptr++];
      }
      else {
	if (buffer.length() > 0) {
	  break;
	}
	else {
	  return ListReader.EOS;
	}
      }

      // if not newline
      if (c != '\n') { 
	buffer.append(c);
	if (c == '(') {
	  leftpar++;
	}
	else if (c == ')') {
	  rightpar++;
	  if (leftpar == rightpar) {
	    break;
	  }
	  else if (leftpar < rightpar) {
	    break;
	  }
	}
      }
      // else if newline
      else {
	// do nothing
	;
      }
    }

    return buffer.toString();
  }
}
