package kudos.ide.form;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.undo.*;
import javax.swing.event.*;
import javax.swing.text.*;

import kudos.ide.io.*;
import kudos.ide.form.*;
import kudos.ide.buffer.*;
import kudos.ide.tools.*;
import kudos.ide.action.menu.*;
import kudos.ide.action.key.*;
import kudos.ide.form.textarea.*;
import kudos.ide.listener.Listener;

public class MainControl
{
  private static int mainViewCount;

  public UndoManager undo = null;
  private static Hashtable actionHash = null;

  private static EditPopupMenu popup = null;

  private static SplashWindow splashWindow = null;
  private static MainView mainView = null;
  private static MainMenuBar mainMenuBar = null;
  private static MainToolBar mainToolBar = null;
  private static StatusBar mainStatusBar = null;
  private KUDOSActions kudosActions[] = null;

  private static NewFileAction newFileAction = null;
  private static OpenFileAction openFileAction = null;
  private static InsertFileAction insertFileAction = null;
  private static SaveAction saveAction = null;
  private static SaveAsAction saveAsAction = null;
  private static ExitAction exitAction = null;
  public UndoAction undoAction = null;
  public RedoAction redoAction = null;
  public static CopyAction copyAction = null;
  public static CutAction cutAction = null;
  public static PasteAction pasteAction = null;
  public static SelectAllAction selectAllAction = null;
  private static ClearAction clearAction = null;
  private static JVMBrowserAction jvmBrowserAction = null;
  private static InterpreterAction interpreterAction = null;
  private static EvaluateBufferAction evaluateBufferAction = null;
  private static EvaluateDefinitionAction evaluateDefinitionAction = null;
  private static EvaluateExpressionAction evaluateExpressionAction = null;
  private static CompileAction compileAction = null;
  private static AboutAction aboutAction = null;
	
  private static Buffer buffer = null;
  private static Buffer oldBuffer = null;
  private static InputHandler inputHandler = null;

  public KUDOSActions[] getKudosActions()
  { 
    KUDOSActions kudosActions[] = new KUDOSActions[20];
    kudosActions[0] = newFileAction;
    kudosActions[1] = openFileAction;
    kudosActions[2] = insertFileAction;
    kudosActions[3] = saveAction;
    kudosActions[4] = saveAsAction;
    kudosActions[5] = exitAction;
    kudosActions[6] = undoAction;
    kudosActions[7] = redoAction;
    kudosActions[8] = copyAction;
    kudosActions[9] = cutAction;
    kudosActions[10] = pasteAction;
    kudosActions[11] = selectAllAction;
    kudosActions[12] = clearAction;
    kudosActions[13] = jvmBrowserAction;
    kudosActions[14] = interpreterAction;
    kudosActions[15] = evaluateBufferAction;
    kudosActions[16] = evaluateDefinitionAction;
    kudosActions[17] = evaluateExpressionAction;
    kudosActions[18] = compileAction;
    kudosActions[19] = aboutAction;

    return kudosActions;
  }

  public MainControl()
  {
      //    splashWindow = new SplashWindow(new Frame());
    initData();
    makeAction();
    buildMainView();
    buildControls();
    selectData();
  }

  private void selectData()
  {
    newFile(mainView);
    VirtualFileSystemManager.start();
    mainView.getInterpreterView().getListener().setMessage(Listener.START);
    mainView.getInterpreterView().getListener().repl();
  }

  private void initData()
  {
      //    splashWindow.setStatus("KUDOS initData Loading...");
    inputHandler = new DefaultInputHandler(null);

    buffer = new Buffer(null,Tools.getNewFileName(),false,true,false,null);
  }

  private void makeAction()
  {
      //    splashWindow.setStatus("KUDOS MenuAction Loading...");
    undo = new UndoManager();
    newFileAction = new NewFileAction(this, "NEW_FILE");
    openFileAction = new OpenFileAction(this, "OPEN_FILE");
    insertFileAction = new InsertFileAction(this, "INSERT_FILE");
    saveAction = new SaveAction(this, "SAVE");
    saveAsAction = new SaveAsAction(this, "SAVE_AS");
    exitAction = new ExitAction(this, "EXIT");
    undoAction = new UndoAction(this, "UNDO");
    redoAction = new RedoAction(this, "REDO");
    copyAction = new CopyAction(this, "COPY");
    cutAction = new CutAction(this, "CUT");
    pasteAction = new PasteAction(this, "PASTE");
    selectAllAction = new SelectAllAction(this, "SELECT_ALL");
    clearAction = new ClearAction(this, "CLEAR");
    jvmBrowserAction = new JVMBrowserAction(this, "JVM_BROWSER");
    interpreterAction = new InterpreterAction(this, "INTERPRETER");
    evaluateBufferAction = new EvaluateBufferAction(this, "EVALUATE_BUFFER");
    evaluateDefinitionAction = new EvaluateDefinitionAction(this, "EVALUATE_DEFINITION");
    evaluateExpressionAction = new EvaluateExpressionAction(this, "EVALUATE_EXPRESSION");
    compileAction = new CompileAction(this, "COMPILE");
    aboutAction = new AboutAction(this, "ABOUT");
		
    actionHash = new Hashtable();

    addAction("InsertChar", null);
    addAction("Up", "UP");
    addAction("Down", "DOWN");
    addAction("Left", "LEFT");
    addAction("Right", "RIGHT");
    addAction("Enter", "ENTER");
    addAction("Esc", "ESCAPE");
    addAction("Delete", "DELETE");
    addAction("BackSpace", "BACK_SPACE");
    addAction("Tab", "TAB");
    addAction("Home", "HOME");
    addAction("End", "END");
    addAction("PageUp", "PAGE_UP");
    addAction("PageDown", "PAGE_DOWN");
    addAction("LeftWord", "C+LEFT");
    addAction("RightWord", "C+RIGHT");
    addAction("SelectUp", "S+UP");
    addAction("SelectDown", "S+DOWN");
    addAction("SelectLeft", "S+LEFT");
    addAction("SelectRight", "S+RIGHT");
    addAction("SelectHome", "S+HOME");
    addAction("SelectEnd", "S+END");
    addAction("SelectPageUp", "S+PAGE_UP");
    addAction("SelectPageDown", "S+PAGE_DOWN");
    addAction("BackTab", "S+TAB");
    addAction("SelectLeftWord", "CS+LEFT");
    addAction("SelectRightWord", "CS+RIGHT");
    addAction("Cut", "C+x");
    addAction("Copy", "C+c");
    addAction("Paste", "C+v");
    addAction("SelectAll", "C+a");
    addAction("Undo", "C+z");
    addAction("Redo", "C+y");
    addAction("NewFile", "C+n");
    addAction("OpenFile", "C+o");
    addAction("InsertFile", null);
    addAction("Save", "C+s");
    addAction("SaveAs", null);
    addAction("LocateBracket", "C+]");

    inputHandler.setInputAction(getAction("InsertChar"));
    popup = new EditPopupMenu(this);
  }

  private static void addAction(String name, String shortcut)
  {
    EditAction.Wrapper action = new EditAction.Wrapper(name);
    actionHash.put(name,action);

    if (shortcut != null)
    {
      inputHandler.addKeyBinding(shortcut,action);
    }
  }

  public EditPopupMenu getEditPopupMenu()
  {
    return popup;
  }

  public static EditAction getAction(String action)
  {
    return (EditAction)actionHash.get(action);
  }

  private void buildMainView()
  {
    mainView = new MainView(this);
  }

  public static MainView getMainView()
  {
    return mainView;
  }

  public static InputHandler getInputHandler()
  {
    return inputHandler;
  }

  public static Buffer getBuffer()
  {
    return buffer;
  }

  public static Buffer getOldBuffer()
  {
    return oldBuffer;
  }

  public void setOldBuffer(Buffer oldBuffer)
  {
    this.oldBuffer = oldBuffer;
  }

  private void buildControls()
  {
      //    splashWindow.setStatus("KUDOS Menu Loading...");
    mainMenuBar = mainView.makeMenu();

    //    splashWindow.setStatus("KUDOS ToolBar Loading...");
    mainToolBar = mainView.makeToolBar();

    //    splashWindow.setStatus("KUDOS StatusBar Loading...");
    mainStatusBar = mainView.makeStatusBar();

    //    splashWindow.setStatus("KUDOS SpiltLine Loading...");
    mainView.makeSpiltView();

    //    splashWindow.setStatus("KUDOS JVM Browser Loading...");
    mainView.makeJVMBrowser();

    //    splashWindow.setStatus("KUDOS Interpreter Interface Loading...");
    mainView.makeInterpreterView();

    //    splashWindow.setStatus("KUDOS EditView Loading...");
    mainView.makeEditTextArea();
	
    //    splashWindow.setStatus("Starting " + Tools.getProgramName() + "...");
    mainView.setVisible(true);
    //    splashWindow.dispose();
    mainView.setBuffer(buffer);		
  }

  public void doProcess(ActionEvent e, KUDOSActions action)
  {
    String command = action.getCommand();

    if(command.equalsIgnoreCase("NEW_FILE"))
    {
      NewFile newFile = new NewFile();
      newFile.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("OPEN_FILE"))
    {
      OpenFile openFile = new OpenFile();
      openFile.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("INSERT_FILE"))
    {
      InsertFile insertFile = new InsertFile();
      insertFile.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("SAVE"))
    {
      Save save = new Save();
      save.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("SAVE_AS"))
    {
      SaveAs saveAs = new SaveAs();
      saveAs.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("EXIT"))
    {
      exit(mainView, mainView.getBuffer());
    }
	
    else if(command.equalsIgnoreCase("UNDO"))
    {
      Undo undo = new Undo();
      undo.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("REDO"))
    {
      Redo redo = new Redo();
      redo.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("COPY"))
    {
      Copy copy = new Copy();
      copy.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("CUT"))
    {
      Cut cut = new Cut();
      cut.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("PASTE"))
    {
      Paste paste = new Paste();
      paste.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("SELECT_ALL"))
    {
      SelectAll selectAll = new SelectAll();
      selectAll.actionPerformed(e);
    }
    else if(command.equalsIgnoreCase("CLEAR"))
    {
      Clear clear = new Clear();
      clear.actionPerformed(e);
    }

    else if(command.equalsIgnoreCase("JVM_BROWSER"))
    {
      doToggleJVMBrowser(e);
    }
    else if(command.equalsIgnoreCase("INTERPRETER"))
    {
      doToggleInterpreter(e);
    }

    else if(command.equalsIgnoreCase("EVALUATE_BUFFER"))
    {
      evaluateBuffer();
    }
    else if(command.equalsIgnoreCase("EVALUATE_DEFINITION"))
    {
      evaluateDefinition();
    }
    else if(command.equalsIgnoreCase("EVALUATE_EXPRESSION"))
    {
      evaluateExpression();
    }
    else if(command.equalsIgnoreCase("COMPILE"))
    {
    }

    else if(command.equalsIgnoreCase("ABOUT"))
    {
    }
  }

  private void evaluateBuffer()
  {
    String text = mainView.getEditTextArea().getText(0,
	mainView.getEditTextArea().getDocumentLength());
    removeCommentary(0, text);
    evaluate(EvaluateData.getText());
  }
	
  private void evaluateDefinition()
  {
    int caret = getMainView().getEditTextArea().getCaretPosition();
    String text = getMainView().getEditTextArea().getText();
    removeCommentary(caret, text);
    text = EvaluateData.getText();
    parenthesis(EvaluateData.getStart(), text);
    String parText = "";
    for(;;) 
    {
      if(EvaluateData.getText() == "")
      {
	evaluate(parText);
	break;
      }
      else
      {
	parText = EvaluateData.getText();
	parenthesis(EvaluateData.getStart(), text);
      }
    }
  }

  private void evaluateExpression()
  {
    int caret = getMainView().getEditTextArea().getCaretPosition();
    String text = getMainView().getEditTextArea().getText();
    removeCommentary(caret, text);
    parenthesis(EvaluateData.getStart(), EvaluateData.getText());
    evaluate(EvaluateData.getText());
  }

  private void removeCommentary(int start, String text)
  {
    char c[] = text.toCharArray();
    boolean isAppend = true;
    String coText = "";
    for(int i = 0;i <= text.length() - 1;i++)
    {
      if(c[i] == ';')
	isAppend = false;

			
      if(isAppend)
	coText += String.valueOf(c[i]);
      else if(c[i] != '\n' || c[i] != '\t')
	coText += " ";
      else if(i <= start || 
	      (i != 0 && coText.charAt(coText.length()) == ' ' && c[i] == ' '))
	start--;

      if(c[i] == '\n')
	isAppend = true;
    }
    EvaluateData.setData(start, coText);
  }

  private void parenthesis(int start, String text)
  {
    int i = 0;
    try
    {
      int newStart = 0;
      if(text != null)
      {
	if( start != 0)
	  i = start - 1;
	char c[] = text.toCharArray();
	int rem = 0;
	int par = -1;
	boolean isend = false;
	boolean isrem = false;
	text = "";
	for(;;)
	{
	  if(c[i] == '(')
	  {
	    par++;
	    if(par == 0)
	    {
	      newStart = i;
	      par = 0;
	      for(;;)
	      {
		text += String.valueOf(c[i]);
		if(c[i] == '(')
		  par++;				
		else if(c[i] == ')')
		{
		  par--;
		  if(par == 0)
		  {
		    isend = true;
		    break;
		  }
		}
		else if(i == mainView.getEditTextArea().getText().length())
		  break;
		i++;
	      }
	    }
	  }
	  else if(c[i] == ')' && i != start)
	  {
	    par--;
	  }

	  if(i < start && isend)
	    text = "";
	  if(par == 0 && isend)
	    break;
	  if((i == 0) || (i == mainView.getEditTextArea().getText().length()))
	  {
	    text = "";
	    break;
	  }
	  i--;
	}
      }
      else
	text = "";
      EvaluateData.setData(newStart, text);
    }
    catch(ArrayIndexOutOfBoundsException ai)
    {
      text = "";
      EvaluateData.setData(i, text);
    }
  }

  /**
   * changed by cjeong
   */
  private void evaluate(String text)
  {
    if(text == null) {
      text = "";
    }
    ListReader reader = new ListReader(text);
    
    while (true) {
      String list = reader.readList();
      if (list.equals(ListReader.EOS)) {
	break;
      }
      else {
	mainView.getInterpreterView().getListener().isEnter();
	mainView.getInterpreterView().getListener().say(list);
	mainView.getInterpreterView().getListener().readCharecter(list);
	mainView.getInterpreterView().getListener().repl();
      }
    }
  }


  private void doToggleJVMBrowser(ActionEvent e)
  {
    Object obj = e.getSource();
    if (obj instanceof JCheckBoxMenuItem)
    {
      mainToolBar.setStateJVMBrowser(mainMenuBar.getStateJVMBrowser());
    }
    else if (obj instanceof JToggleButton)
    {
      mainMenuBar.setStateJVMBrowser(mainToolBar.getStateJVMBrowser());
    }

    if (mainMenuBar.getStateJVMBrowser())
    {
      mainView.setShowJVMBrowser(true);
    }
    else
    {
      mainView.setShowJVMBrowser(false);
    }
  }

  private void doToggleInterpreter(ActionEvent e)
  {
    Object obj = e.getSource();
    if (obj instanceof JCheckBoxMenuItem)
    {
      mainToolBar.setStateInterpreter(mainMenuBar.getStateInterpreter());
    }
    else if (obj instanceof JToggleButton)
    {
      mainMenuBar.setStateInterpreter(mainToolBar.getStateInterpreter());
    }

    if (mainMenuBar.getStateInterpreter())
    {
      mainView.setShowInterpreter(true);
    }
    else
    {
      mainView.setShowInterpreter(false);
    }
  }

  public void windowClosed()
  {
    System.exit(0);
  }

  public static Buffer newFile(MainView mainView)
  {
    buffer = mainView.getBuffer();
    if(buffer.isDirty())
    {
      boolean isReturn = true;
      int result = 
	JOptionPane.showConfirmDialog(mainView,
	  ("Save changes to " + buffer.toString() + "?"),
	  "File Not Saved", JOptionPane.YES_NO_CANCEL_OPTION,
	  JOptionPane.WARNING_MESSAGE);
      if(result == JOptionPane.YES_OPTION)
      {
	buffer.save(mainView,null);
	isReturn = false;
      }

      if(result == JOptionPane.NO_OPTION)
	isReturn = false;

      if(isReturn)
	return buffer;
    }
    try
    {
      buffer.remove(0,buffer.getLength());
    }
    catch(BadLocationException bl){}

    buffer.setDirty(false);
    return openFile(mainView,null,Tools.getNewFileName(),false,true,null);
  }

  public static Buffer openFile(MainView mainView, String parent,
				String path, boolean readOnly, boolean newFile, Hashtable props)
  {
    mainView.setTitle(Tools.getProgramName() + " - [" + path + "]");

    buffer = new Buffer(mainView,path,readOnly,newFile,false,props);

    if(!buffer.load(mainView,false))
      return null;

    return buffer;
  }

  public static Buffer insertFile(MainView mainView, String path)
  {
    oldBuffer = mainView.getEditTextArea().getBuffer();

    return openFile(mainView,null,path,false,false,null);
  }

  public void exit(MainView mainView, Buffer buffer)
  {
    if(buffer.isDirty())
    {
      int result = JOptionPane.showConfirmDialog(mainView,
						 ("Save changes to " + buffer.toString() + "?"),
						 "File Not Saved", JOptionPane.YES_NO_CANCEL_OPTION,
						 JOptionPane.WARNING_MESSAGE);
      if(result == JOptionPane.YES_OPTION)
      {
	buffer.save(mainView,null);
      }
      if(result == JOptionPane.NO_OPTION)
	windowClosed();
      if(result == JOptionPane.CANCEL_OPTION)
	return;
    }
    windowClosed();
  }

  public static class EvaluateData
  {
    private static int start;
    private static String text;

    public static int getStart()
    {
      return start;
    }

    public static String getText()
    {
      return text;
    }

    public static void setData(int start, String text)
    {
      EvaluateData.start = start;
      EvaluateData.text = text;
    }
  }
}
