package kudos.ide.form;

import javax.swing.*;
import javax.swing.border.*;

import kudos.ide.action.menu.*;

public class MainToolBar extends JToolBar
{
    private MainControl mainControl;
    private KUDOSActions kudosActions[];
    private JToggleButton jvmBrowserButton = null;
    private JToggleButton interpreterButton = null;

    public MainToolBar(MainControl mainControl)
    {
	this.mainControl = mainControl;
	kudosActions = mainControl.getKudosActions();
	setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	makeTools();
    }

    public boolean getStateJVMBrowser()
    {
	return jvmBrowserButton.isSelected();
    }
    public void setStateJVMBrowser(boolean state)
    {
	jvmBrowserButton.setSelected(state);
    }
    public boolean getStateInterpreter()
    {
	return interpreterButton.isSelected();
    }
    public void setStateInterpreter(boolean state)
    {
	interpreterButton.setSelected(state);
    }

    public void makeTools()
    {
	for (int i=0; i<kudosActions.length; i++)
	{
	    String command = kudosActions[i].getCommand();

	    if(command.equalsIgnoreCase("NEW_FILE"))
	    {
		JButton newFileButton = new JButton(kudosActions[i]);
		newFileButton.setText("");
		newFileButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(newFileButton);
	    }
	    else if(command.equalsIgnoreCase("OPEN_FILE"))
	    {
		JButton openFileButton = new JButton(kudosActions[i]);
		openFileButton.setText("");
		openFileButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(openFileButton);
	    }
	    else if(command.equalsIgnoreCase("SAVE"))
	    {
		JButton saveButton = new JButton(kudosActions[i]);
		saveButton.setText("");
		saveButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(saveButton);
		addSeparator();
	    }
	    else if(command.equalsIgnoreCase("UNDO"))
	    {
		JButton undoButton = new JButton(kudosActions[i]);
		undoButton.setText("");
		undoButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(undoButton);
	    }
	    else if(command.equalsIgnoreCase("REDO"))
	    {
		JButton redoButton = new JButton(kudosActions[i]);
		redoButton.setText("");
		redoButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(redoButton);
		addSeparator();
	    }
	    else if(command.equalsIgnoreCase("COPY"))
	    {
		JButton copyButton = new JButton(kudosActions[i]);
		copyButton.setDefaultCapable(true);
		copyButton.setText("");
		copyButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(copyButton);
	    }
	    else if(command.equalsIgnoreCase("CUT"))
	    {
		JButton cutButton = new JButton(kudosActions[i]);
		cutButton.setText("");
		cutButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(cutButton);
	    }
	    else if(command.equalsIgnoreCase("PASTE"))
	    {
		JButton pasteButton = new JButton(kudosActions[i]);
		pasteButton.setText("");
		pasteButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(pasteButton);
		addSeparator();
	    }
	    else if(command.equalsIgnoreCase("JVM_BROWSER"))
	    {
		jvmBrowserButton = new JToggleButton(kudosActions[i]);
		jvmBrowserButton.setText("");
		jvmBrowserButton.setSelected(true);
		jvmBrowserButton.setBorder(
		    BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(1, 1, 1, 1)
			));
		jvmBrowserButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(jvmBrowserButton);
	    }
	    else if(command.equalsIgnoreCase("INTERPRETER"))
	    {
		interpreterButton = new JToggleButton(kudosActions[i]);
		interpreterButton.setText("");
		interpreterButton.setSelected(true);
		interpreterButton.setBorder(
		    BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(1, 1, 1, 1)
			));
		interpreterButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(interpreterButton);
		addSeparator();
	    }
	    else if(command.equalsIgnoreCase("EVALUATE_BUFFER"))
	    {
		JButton evaluateBufferButton = new JButton(kudosActions[i]);
		evaluateBufferButton.setText("");
		evaluateBufferButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(evaluateBufferButton);
	    }
	    else if(command.equalsIgnoreCase("EVALUATE_DEFINITION"))
	    {
		JButton evaluateDefinitionAction = new JButton(kudosActions[i]);
		evaluateDefinitionAction.setText("");
		evaluateDefinitionAction.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(evaluateDefinitionAction);
	    }
	    else if(command.equalsIgnoreCase("EVALUATE_EXPRESSION"))
	    {
		JButton evaluateExptessionButton = new JButton(kudosActions[i]);
		evaluateExptessionButton.setText("");
		evaluateExptessionButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(evaluateExptessionButton);
	    }
	    else if(command.equalsIgnoreCase("COMPILE"))
	    {
		JButton compileButton = new JButton(kudosActions[i]);
		compileButton.setText("");
		compileButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(compileButton);
		addSeparator();
	    }
	    else if(command.equalsIgnoreCase("ABOUT"))
	    {
		JButton aboutButton = new JButton(kudosActions[i]);
		aboutButton.setText("");
		aboutButton.setToolTipText(kudosActions[i].TOOLTIP_TEXT);
		add(aboutButton);
		
		for (int j=0; j<kudosActions.length; j++)
		{
		    String exit = kudosActions[j].getCommand();
	
		    if(exit.equalsIgnoreCase("EXIT"))
		    {
			JButton exitButton = new JButton(kudosActions[j]);
			exitButton.setText("");
			exitButton.setToolTipText(kudosActions[j].TOOLTIP_TEXT);
			add(exitButton);
		    }
		}
	    }
	}
    }
}
