package kudos.ide.form;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import kudos.ide.form.*;
import kudos.ide.buffer.*;
import kudos.ide.tools.*;
import kudos.ide.action.menu.*;
import kudos.ide.form.textarea.*;

public class MainView extends JFrame
{
  private int waitCount;
  private int splitPaneSize;
  private int oldWindowSize;
  private int oldInterpreterSplitPaneSize;
  private int oldSplitPaneSize;
  private boolean closed;

  private Container contentPane = null;
  private Dimension frameSize = null;
  private JPanel editPanel = null;
  private JPanel jvmBrowserPanel = null;
  private JPanel interpreterPanel = null;
  private JSplitPane jvmBrowserSplitPane = null;
  private JSplitPane InterpreterSplitPane = null;
  private JSplitPane splitPane = null;
  private JScrollPane interpreterScrollPane = null;

  private MainControl mainControl = null;
  private EditTextArea editTextArea = null;
  public JVMBrowserView jvmBrowserView = null;
  public JVMHelpTextBox jvmHelpTextBox = null;
  private InterpreterView interpreterView = null;
  private InputHandler inputHandler = null;
  //	private GlassPane glassPane = null;

  public MainView(MainControl mainControl)
  {
    super(Tools.getProgramName());
    this.mainControl = mainControl;
    contentPane = getContentPane();
    contentPane.setLayout(new BorderLayout());
    inputHandler = new DefaultInputHandler(this,(DefaultInputHandler)
					   MainControl.getInputHandler());
    editPanel = new JPanel(new BorderLayout());
    jvmBrowserPanel = new JPanel(new BorderLayout());
    interpreterPanel = new JPanel(new BorderLayout());

    Dimension screenSize = Tools.getScreenSize();
    setBounds(0, 0, screenSize.width * 90 / 100, screenSize.height * 90 / 100);
    frameSize = getSize();
    setLocation((screenSize.width  - frameSize.width ) / 2, 
		(screenSize.height - frameSize.height) / 2);
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

    addWindowListener(new WindowHandler());
  }

  public void setShowJVMBrowser(boolean b)
  {
    jvmBrowserPanel.setVisible(b);
    jvmBrowserSplitPane.setEnabled(b);
    jvmBrowserSplitPane.setDividerLocation(getSize().width * 30 / 100);
  }
	
  public void setShowInterpreter(boolean b)
  {
    interpreterPanel.setVisible(b);
    InterpreterSplitPane.setEnabled(b);
    InterpreterSplitPane.setDividerLocation(getSize().height * 55 / 100);
  }

  public MainMenuBar makeMenu()
  {
    MainMenuBar mainMenuBar = new MainMenuBar(mainControl);
    setJMenuBar(mainMenuBar);
    return mainMenuBar;
  }

  public MainToolBar makeToolBar()
  {
    MainToolBar mainToolBar = new MainToolBar(mainControl);
    contentPane.add(mainToolBar, BorderLayout.NORTH);
    return mainToolBar;
  }
	
  public StatusBar makeStatusBar()
  {
    StatusBar statusBar = new StatusBar(frameSize.width);
    contentPane.add(statusBar, BorderLayout.SOUTH);
    return statusBar;
  }
	
  public void setBuffer(Buffer buffer)
  {
    editTextArea.setBuffer(buffer);
  }

  public Buffer getBuffer()
  {
    return editTextArea.getBuffer();
  }

  public InterpreterView getInterpreterView()
  {
    return interpreterView; 
  }

  public void makeSpiltView()
  {
    jvmBrowserSplitPane = new JSplitPane (JSplitPane.HORIZONTAL_SPLIT);
    JPanel mainPanel = new JPanel(new BorderLayout());
    jvmBrowserSplitPane.setDividerLocation(getSize().width * 20 / 100);
    jvmBrowserSplitPane.setOneTouchExpandable (true);

    jvmBrowserSplitPane.setRightComponent(mainPanel);
    jvmBrowserSplitPane.setLeftComponent(jvmBrowserPanel);

    contentPane.add(jvmBrowserSplitPane , BorderLayout.CENTER);

    InterpreterSplitPane = new JSplitPane (JSplitPane.VERTICAL_SPLIT);
    InterpreterSplitPane.setDividerLocation(getSize().height * 60 / 100);
    oldInterpreterSplitPaneSize = 
      getSize().height - InterpreterSplitPane.getDividerLocation();
    oldWindowSize = getSize().height;
    InterpreterSplitPane.setOneTouchExpandable (true);

    InterpreterSplitPane.setTopComponent(editPanel);
    InterpreterSplitPane.setBottomComponent(interpreterPanel);

    mainPanel.add(InterpreterSplitPane , BorderLayout.CENTER);
  }

  // use for resizing the top-to-bottom splitter
  public void setInterpreterSplitPaneDividerLocation()
  {
    int newSize = getSize().height - InterpreterSplitPane.getDividerLocation();

    if(oldInterpreterSplitPaneSize != newSize)
    {
      InterpreterSplitPane.setDividerLocation(getSize().height - 
					      oldInterpreterSplitPaneSize);
    }
  }

  // used for resizing the editor-interpreter splitter
  public void setOldInterpreterSplitPaneSize()
  {
    if(oldWindowSize == getSize().height)
      oldInterpreterSplitPaneSize = 
	getSize().height - InterpreterSplitPane.getDividerLocation();
    oldWindowSize = getSize().height;
  }

  public void makeJVMBrowser()
  {
    JPanel panel = new JPanel(new BorderLayout());
    jvmBrowserView = new JVMBrowserView(this);
    //jvmBrowserView = JVMBrowserView.getInstance(this);
    jvmHelpTextBox = new JVMHelpTextBox(this);
    JScrollPane scrollPane = new JScrollPane(jvmHelpTextBox);
    panel.add(scrollPane, BorderLayout.CENTER);

    splitPane = new JSplitPane (JSplitPane.VERTICAL_SPLIT);
    splitPane.setDividerLocation(getSize().height * 70 / 100);
    splitPaneSize = getSize().height - splitPane.getDividerLocation();
    // cjeongbegin
    splitPane.setOneTouchExpandable(true);
    splitPane.setEnabled(true);
    // cjeongend
    splitPane.setTopComponent(jvmBrowserView);
    splitPane.setBottomComponent(panel);

    jvmBrowserPanel.add(splitPane, BorderLayout.CENTER);
  }

  public void setSplitPaneSizeHeight()
  {
    // cjeongbegin
    //int newSize = 
    //      getSize().height - splitPane.getDividerLocation();

    if(splitPaneSize == getSize().height) {
      splitPane.setDividerLocation(getSize().height - 
				   splitPaneSize);
      splitPaneSize = getSize().height;
    }
    // cjeongend
    //splitPane.setDividerLocation(getSize().height - splitPaneSize);
  }

  public void makeInterpreterView()
  {
    interpreterView = new InterpreterView(this);
    interpreterView.setMargin(new Insets(5,5,5,5));
    interpreterScrollPane = new JScrollPane(interpreterView);
    interpreterPanel.add(interpreterScrollPane, BorderLayout.CENTER);
  }

  public JScrollPane getInterpreterScrollPane()
  {
    return interpreterScrollPane;
  }

  public void makeEditTextArea()
  {
    editTextArea = new EditTextArea(mainControl);
    editPanel.add(editTextArea, BorderLayout.CENTER);
  }

  public final boolean isClosed()
  {
    return closed;
  }

  public final InputHandler getInputHandler()
  {
    return inputHandler;
  }

  public EditTextArea getEditTextArea()
  {
    return editTextArea;
  }

  public MainControl getMainControl()
  {
    return mainControl;
  }

  public void windowClose()
  {
    mainControl.exit(this, getBuffer());
  }

  private class WindowHandler extends WindowAdapter
  {
    public void windowClosing(WindowEvent evt)
    {
      windowClose();
    }
  }

  /*	private class GlassPane extends JComponent
	{
	GlassPane()
	{
	GlassPane.this.enableEvents(AWTEvent.KEY_EVENT_MASK);
	GlassPane.this.enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	GlassPane.this.setOpaque(false);
	GlassPane.this.setCursor(Cursor.getPredefinedCursor(
	Cursor.WAIT_CURSOR));
	}
	}*/
}
