package kudos.ide.form.textarea;

import java.awt.event.*;
import java.util.*;
import javax.swing.*;

import kudos.ide.form.*;

public class DefaultInputHandler extends InputHandler
{
  private Hashtable bindings = null;
  private Hashtable currentBindings = null;

  public DefaultInputHandler(MainView mainView)
  {
    super(mainView);

    bindings = currentBindings = new Hashtable();
  }

  public DefaultInputHandler(MainView mainView, DefaultInputHandler copy)
  {
    super(mainView);

    bindings = currentBindings = copy.bindings;
    inputAction = copy.inputAction;
  }

  public void addKeyBinding(String keyBinding, EditAction action)
  {
    Hashtable current = bindings;

    StringTokenizer st = new StringTokenizer(keyBinding);
    while(st.hasMoreTokens())
    {
      KeyStroke keyStroke = parseKeyStroke(st.nextToken());
      if(keyStroke == null)
	return;

      if(st.hasMoreTokens())
      {
	Object o = current.get(keyStroke);
	if(o instanceof Hashtable)
	  current = (Hashtable)o;
	else
	{
	  o = new Hashtable();
	  current.put(keyStroke,o);
	  current = (Hashtable)o;
	}
      }
      else
	current.put(keyStroke,action);
    }
  }

  public static KeyStroke parseKeyStroke(String keyStroke)
  {
    if(keyStroke == null)
      return null;
    int modifiers = 0;
    int index = keyStroke.indexOf('+');
    if(index != -1)
    {
      for(int i = 0; i < index; i++)
      {
	switch(Character.toUpperCase(keyStroke
				     .charAt(i)))
	{
	case 'A':
	  modifiers |= InputEvent.ALT_MASK;
	  break;
	case 'C':
	  modifiers |= InputEvent.CTRL_MASK;
	  break;
	case 'M':
	  modifiers |= InputEvent.META_MASK;
	  break;
	case 'S':
	  modifiers |= InputEvent.SHIFT_MASK;
	  break;
	}
      }
    }
    String key = keyStroke.substring(index + 1);
    if(key.length() == 1)
    {
      char ch = Character.toUpperCase(key.charAt(0));
      if(modifiers == 0)
	return KeyStroke.getKeyStroke(ch);
      else
	return KeyStroke.getKeyStroke(ch,modifiers);
    }
    else if(key.length() == 0)
    {
      return null;
    }
    else
    {
      int ch;

      try
      {
	ch = KeyEvent.class.getField("VK_".concat(key))
	  .getInt(null);
      }
      catch(Exception e)
      {
	return null;
      }

      return KeyStroke.getKeyStroke(ch,modifiers);
    }
  }

  public void keyPressed(KeyEvent evt)
  {
    int keyCode = evt.getKeyCode();
    int modifiers = evt.getModifiers();

    if(keyCode == KeyEvent.VK_CONTROL ||
       keyCode == KeyEvent.VK_SHIFT ||
       keyCode == KeyEvent.VK_ALT ||
       keyCode == KeyEvent.VK_META)
      return;

    if((modifiers & ~KeyEvent.SHIFT_MASK) != 0
       || evt.isActionKey()
       || keyCode == KeyEvent.VK_BACK_SPACE
       || keyCode == KeyEvent.VK_DELETE
       || keyCode == KeyEvent.VK_ENTER
       || keyCode == KeyEvent.VK_TAB
       || keyCode == KeyEvent.VK_ESCAPE)
    {
      KeyStroke keyStroke = KeyStroke.getKeyStroke(keyCode,modifiers);
      Object o = currentBindings.get(keyStroke);

      if(o instanceof EditAction)
      {
	currentBindings = bindings;

	executeAction(((EditAction)o),
		      evt.getSource(),null);

	evt.consume();
	return;
      }
    }
  }

  public void keyTyped(KeyEvent evt)
  {
    int modifiers = evt.getModifiers();
    char c = evt.getKeyChar();

    if(c != KeyEvent.CHAR_UNDEFINED &&
       (modifiers & KeyEvent.ALT_MASK) == 0)
    {
      if(c >= 0x20 && c != 0x7f)
      {
	executeAction(inputAction,evt.getSource(),
		      String.valueOf(evt.getKeyChar()));
	evt.consume();
	repeatCount = 0;
	repeat = false;
      }
    }
  }
}
