package kudos.ide.form.textarea;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.undo.*;

import kudos.ide.action.menu.*;
import kudos.ide.syntax.*;
import kudos.ide.form.*;
import kudos.ide.buffer.*;

public class EditTextArea extends JComponent
{
  private TextAreaPainter textAreaPainter = null;

  private boolean rectSelect;
  private boolean overwrite;
  private boolean biasLeft;
  private boolean caretBlinks;
  private boolean caretVisible;
  private boolean editable;
  private boolean blink;
  private boolean scrollBarsInitialized;
  private int firstLine;
  private int horizontalOffset;
  private int selectionStartLine;
  private int selectionEndLine;
  private int selectionStart;
  private int selectionEnd;
  private int magicCaret;
  private int visibleLines;
  private int electricScroll;
  private int clickCount;
  private int dragStartLine;
  private int dragStartOffset;
  private int documentCaret;

  private JScrollBar vertical = null;
  private JScrollBar horizontal = null;
  private EventListenerList listenerList = null;
  private static Timer caretTimer = null;
  Segment lineSegment = null;

  private Buffer buffer = null;
  private Buffer recentBuffer = null;
  private SyntaxDocument document = null;
  private DocumentHandler documentHandler = null;
  private static EditTextArea focusedComponent = null;

  private EditPopupMenu popup = null;

  public EditTextArea(MainControl mainControl)
  {
    textAreaPainter = new TextAreaPainter(this);
    documentHandler = new DocumentHandler();
    listenerList = new EventListenerList();
    lineSegment = new Segment();
    blink = true;
    popup = mainControl.getEditPopupMenu();
    setBuffer(MainControl.getBuffer());

    JPanel panel = new JPanel();
    panel.setBackground(Color.white);
    setLayout(new ScrollLayout());
    add(CENTER,textAreaPainter);
    add(LEFT,panel);
    add(RIGHT,vertical = new JScrollBar(JScrollBar.VERTICAL));
    add(BOTTOM,horizontal = new JScrollBar(JScrollBar.HORIZONTAL));

    vertical.addAdjustmentListener(new AdjustHandler());
    horizontal.addAdjustmentListener(new AdjustHandler());
    textAreaPainter.addComponentListener(new ComponentHandler());
    textAreaPainter.addMouseListener(new MouseHandler());
    textAreaPainter.addMouseMotionListener(new DragHandler());
    addFocusListener(new FocusHandler());

    caretVisible = true;
    editable = true;
  }

  /*	private void initLineNumber()
	{
	lineNumber.setLineNumberWidth(192);
	lineNumber.setBorder(4, Color.decode("#808080"),
	getTextAreaPainter().getBackground());
	lineNumber.setCollapsed(true);
	lineNumber.setLineNumberingEnabled(true);
	}
  */
  private void updateScrollBars()
  {
    if(vertical != null && visibleLines != 0)
    {
      int lineCount = getLineCount();
      if(firstLine < 0)
      {
	setFirstLine(0);
	return;
      }
      else if(lineCount < firstLine + visibleLines)
      {
	int newFirstLine = Math.max(0,lineCount - visibleLines);
	if(newFirstLine != firstLine)
	{
	  setFirstLine(newFirstLine);
	  return;
	}
      }

      vertical.setValues(firstLine,visibleLines,0,getLineCount());
      vertical.setUnitIncrement(2);
      vertical.setBlockIncrement(visibleLines);
    }

    int width = textAreaPainter.getWidth();
    if(horizontal != null && width != 0)
    {
      horizontal.setValues(-horizontalOffset,width,0,width * 5);
      horizontal.setUnitIncrement(textAreaPainter.getFontMetrics()
				  .charWidth('w'));
      horizontal.setBlockIncrement(width / 2);
    }
  }

  private final EditPopupMenu getRightClickPopup()
  {
    return popup;
  }

  private final void setRightClickPopup(EditPopupMenu popup)
  {
    this.popup = popup;
  }

  public final void selectAll()
  {
    select(0,getDocumentLength());
  }

  public final void clear()
  {
    try
    {
      buffer.remove(0,getDocumentLength());
    }
    catch(BadLocationException bl){}
  }

  public final int getElectricScroll()
  {
    return electricScroll;
  }

  public final int getVisibleLines()
  {
    return visibleLines;
  }

  public final void selectNone()
  {
    select(getCaretPosition(),getCaretPosition());
  }

  public final void setCaretPosition(int caret)
  {
    select(caret,caret);
  }

  public final void setDocumentCaret(int documentCaret)
  {
    this.documentCaret = documentCaret;
  }

  public final int getMagicCaretPosition()
  {
    return magicCaret;
  }

  public final TextAreaPainter getTextAreaPainter()
  {
    return textAreaPainter;
  }

  public void setBuffer(Buffer buffer)
  {

    if(this.buffer == buffer)
      return;

    if(buffer.isClosed())
      throw new InternalError(buffer + " has been closed");

    recentBuffer = this.buffer;
    this.buffer = buffer;

    setDocument(buffer);
  }

  private void setDocument(SyntaxDocument document)
  {
    if(this.document == document)
      return;
    if(this.document != null)
      this.document.removeDocumentListener(documentHandler);
    this.document = document;

    document.addDocumentListener(documentHandler);

    select(0,0);
    updateScrollBars();
    textAreaPainter.repaint();
  }

  private void setCaretVisible(boolean caretVisible)
  {
    this.caretVisible = caretVisible;
    blink = true;

    textAreaPainter.invalidateSelectedLines();
  }

  public final Buffer getBuffer()
  {
    return buffer;
  }

  public final boolean isEditable()
  {
    return editable;
  }

  public final boolean isCaretVisible()
  {
    return blink && caretVisible;
  }

  public int getLineStartOffset(int line)
  {
    Element lineElement = document.getDefaultRootElement()
      .getElement(line);
    return lineElement.getStartOffset();
  }

  public int lineToY(int line)
  {
    FontMetrics fm = textAreaPainter.getFontMetrics();
    return (line - firstLine) * fm.getHeight()
      - (fm.getLeading() + fm.getDescent());
  }

  public final int getDocumentCaret()
  {
    return documentCaret;
  }

  public final int getCaretPosition()
  {
    return (biasLeft ? selectionStart : selectionEnd);
  }

  private final int getMarkLine()
  {
    return (biasLeft ? selectionEndLine : selectionStartLine);
  }

  public final int getCaretLine()
  {
    return (biasLeft ? selectionStartLine : selectionEndLine);
  }

  public final boolean isOverwriteEnabled()
  {
    return overwrite;
  }

  public final int getSelectionStartLine()
  {
    return selectionStartLine;
  }

  public final int getSelectionEndLine()
  {
    return selectionEndLine;
  }

  public final int getFirstLine()
  {
    return firstLine;
  }

  public final int getMarkPosition()
  {
    return (biasLeft ? selectionEnd : selectionStart);
  }

  public final void setMagicCaretPosition(int magicCaret)
  {
    this.magicCaret = magicCaret;
  }

  private int yToLine(int y)
  {
    FontMetrics fm = textAreaPainter.getFontMetrics();
    int height = fm.getHeight();
    return Math.max(0,Math.min(getLineCount() - 1,
			       y / height + firstLine));
  }

  private int xyToOffset(int x, int y)
  {
    int line = yToLine(y);
    int start = getLineStartOffset(line);
    return start + xToOffset(line,x);
  }

  public int xToOffset(int line, int x)
  {
    FontMetrics fm = textAreaPainter.getFontMetrics();

    getLineText(line,lineSegment);

    char[] segmentArray = lineSegment.array;
    int segmentOffset = lineSegment.offset;
    int segmentCount = lineSegment.count;

    int width = horizontalOffset;

    for(int i = 0; i < segmentCount; i++)
    {
      char c = segmentArray[i + segmentOffset];
      int charWidth;
      if(c == '\t')
	charWidth = (int)textAreaPainter.nextTabStop(width,i) - width;
      else
	charWidth = fm.charWidth(c);
      if(textAreaPainter.isBlockCaretEnabled())
      {
	if(x - charWidth <= width)
	  return i;
      }
      else
      {
	if(x - charWidth / 2 <= width)
	  return i;
      }
      width += charWidth;
    }
    return segmentCount;
  }

  public int offsetToX(int line, int offset)
  {
    FontMetrics fm = textAreaPainter.getFontMetrics();

    int x = horizontalOffset;

    getLineText(line,lineSegment);

    lineSegment.count = offset;
    return x + Utilities.getTabbedTextWidth(lineSegment,fm,x,textAreaPainter,0);
  }

  public final int getSelectionStart()
  {
    return selectionStart;
  }

  public final int getSelectionEnd()
  {
    return selectionEnd;
  }

  public final boolean isSelectionRectangular()
  {
    return rectSelect;
  }

  public final int getHorizontalOffset()
  {
    return horizontalOffset;
  }

  public final SyntaxDocument getDocument()
  {
    return document;
  }

  public final int getLineCount()
  {
    return document.getDefaultRootElement().getElementCount();
  }

  protected void processKeyEvent(KeyEvent evt)
  {
    MainView mainView = EditAction.getMainView(evt);

    if(mainView.isClosed())
      return;

    if(mainView.getRootPane().getGlassPane().isVisible())
    {
      super.processKeyEvent(evt);
      return;
    }

    InputHandler inputHandler = mainView.getInputHandler();
    switch(evt.getID())
    {
    case KeyEvent.KEY_TYPED:
      inputHandler.keyTyped(evt);
      break;
    case KeyEvent.KEY_PRESSED:
      inputHandler.keyPressed(evt);
      break;
    case KeyEvent.KEY_RELEASED:
      inputHandler.keyReleased(evt);
      break;
    }

    if(!evt.isConsumed())
      super.processKeyEvent(evt);
  }

  public void overwriteSetSelectedText(String str)
  {
    if(!overwrite || selectionStart != selectionEnd)
    {
      setSelectedText(str);
      return;
    }

    int caret = getCaretPosition();
    int caretLineEnd = getLineEndOffset(getCaretLine());
    if(caretLineEnd - caret <= str.length())
    {
      setSelectedText(str);
      return;
    }
  }

  public int getLineEndOffset(int line)
  {
    Element lineElement = document.getDefaultRootElement().getElement(line);
    if(lineElement == null)
      return -1;
    else
      return lineElement.getEndOffset();
  }

  public final String getLineText(int lineIndex)
  {
    int start = getLineStartOffset(lineIndex);

    return getText(start,getLineEndOffset(lineIndex) - start - 1);
  }

  public final void getLineText(int lineIndex, Segment segment)
  {
    int start = getLineStartOffset(lineIndex);
    getText(start,getLineEndOffset(lineIndex) - start - 1,segment);
  }

  public void setSelectedText(String selectedText)
  {
    setSelectedText(selectedText, selectionStart);
  }

  public void setSelectedText(String selectedText, int selectionStart)
  {
    document.beginCompoundEdit();

    try
    {
      document.remove(selectionStart,
		      selectionEnd - selectionStart);
      if(selectedText != null)
      {
	document.insertString(selectionStart,
			      selectedText,null);
      }
    }
    catch(BadLocationException bl){}
    finally
    {
      document.endCompoundEdit();
    }

    setCaretPosition(selectionEnd);
  }

  public final int getDocumentLength()
  {
    return document.getLength();
  }

  public void select(int start, int end)
  {
    int newStart, newEnd;
    boolean newBias;

    if(start <= end)
    {
      newStart = start;
      newEnd = end;
      newBias = false;
    }
    else
    {
      newStart = end;
      newEnd = start;
      newBias = true;
    }

    if(newStart != selectionStart || newEnd != selectionEnd
       || newBias != biasLeft)
    {
      int newStartLine = getLineOfOffset(newStart);
      int newEndLine = getLineOfOffset(newEnd);

      textAreaPainter.invalidateLineRange(selectionStartLine,selectionEndLine);
      textAreaPainter.invalidateLineRange(newStartLine,newEndLine);

      selectionStart = newStart;
      selectionEnd = newEnd;
      selectionStartLine = newStartLine;
      selectionEndLine = newEndLine;
      biasLeft = newBias;
    }
		
    blink = true;
    caretTimer.restart();

    if(selectionStart == selectionEnd)
      rectSelect = false;

    magicCaret = -1;
    scrollToCaret();
  }

  public boolean scrollToCaret()
  {
    int line = getCaretLine();
    int lineStart = getLineStartOffset(line);
    int offset = Math.max(0,Math.min(getLineLength(line) - 1,getCaretPosition() - lineStart));

    return scrollTo(line,offset);
  }

  public int getLineLength(int line)
  {
    Element lineElement = document.getDefaultRootElement()
      .getElement(line);
    if(lineElement == null)
      return -1;
    else
      return lineElement.getEndOffset() - lineElement.getStartOffset() - 1;
  }

  public final void setSelectionRectangular(boolean rectSelect)
  {
    this.rectSelect = rectSelect;
    textAreaPainter.invalidateSelectedLines();
  }

  public boolean scrollTo(int line, int offset)
  {
    if(visibleLines == 0)
    {
      setFirstLine(Math.max(0,line - electricScroll));
      return true;
    }

    int newFirstLine = firstLine;
    int newHorizontalOffset = horizontalOffset;

    if(line < firstLine + electricScroll)
    {
      newFirstLine = Math.max(0,line - electricScroll);
    }
    else if(line + electricScroll >= firstLine + visibleLines)
    {
      newFirstLine = (line - visibleLines) + electricScroll + 1;
      if(newFirstLine + visibleLines >= getLineCount())
	newFirstLine = getLineCount() - visibleLines;
      if(newFirstLine < 0)
	newFirstLine = 0;
    }

    int x = offsetToX(line,offset);
    int width = textAreaPainter.getFontMetrics().charWidth('w');

    if(x < 0)
    {
      newHorizontalOffset = Math.min(0,horizontalOffset - x + width + 5);
    }
    else if(x + width >= textAreaPainter.getWidth())
    {
      newHorizontalOffset = horizontalOffset +
	(textAreaPainter.getWidth() - x) - width - 5;
    }

    return setOrigin(newFirstLine,newHorizontalOffset);
  }

  public boolean setOrigin(int firstLine, int horizontalOffset)
  {
    boolean vertChanged = false, horizChanged = false;
    int oldFirstLine = this.firstLine;

    if(firstLine != this.firstLine)
    {
      this.firstLine = firstLine;
      vertChanged = true;
    }

    if(horizontalOffset != this.horizontalOffset)
    {
      this.horizontalOffset = horizontalOffset;
      horizChanged = true;
    }

    if(vertChanged || horizChanged)
    {
      updateScrollBars();
      textAreaPainter.repaint();
    }

    return vertChanged || horizChanged;
  }

  public final int getLineOfOffset(int offset)
  {
    return document.getDefaultRootElement().getElementIndex(offset);
  }

  public final String getText()
  {
    try
    {
      return document.getText(0,getDocumentLength());
    }
    catch(BadLocationException bl)
    {
      return null;
    }
  }

  public final String getText(int start, int len)
  {
    try
    {
      return document.getText(start,len);
    }
    catch(BadLocationException bl)
    {
      return null;
    }
  }

  public final void getText(int start, int len, Segment segment)
  {
    try
    {
      document.getText(start,len,segment);
    }
    catch(BadLocationException bl)
    {
      segment.offset = segment.count = 0;
    }
  }

  public final void recalculateVisibleLines()
  {
    if(textAreaPainter == null)
      return;
    int height = textAreaPainter.getHeight();
    int lineHeight = textAreaPainter.getFontMetrics().getHeight();
    visibleLines = height / lineHeight;
    updateScrollBars();
  }

  private void documentChanged(DocumentEvent evt)
  {
    DocumentEvent.ElementChange ch = evt.getChange(
      document.getDefaultRootElement());

    int count;
    if(ch == null)
      count = 0;
    else
      count = ch.getChildrenAdded().length -
	ch.getChildrenRemoved().length;

    int line = getLineOfOffset(evt.getOffset());
    if(count == 0)
    {
      textAreaPainter.invalidateLine(line);
    }
    else if(line < firstLine)
    {
      setFirstLine(firstLine + count);
    }
    else
    {
      textAreaPainter.invalidateLineRange(line,firstLine + visibleLines);
      updateScrollBars();
    }
  }

  public void setFirstLine(int firstLine)
  {
    if(firstLine == this.firstLine)
      return;

    this.firstLine = firstLine;

    if(firstLine != vertical.getValue())
      updateScrollBars();
    textAreaPainter.repaint();
  }

  public void setHorizontalOffset(int horizontalOffset)
  {
    if(horizontalOffset == this.horizontalOffset)
      return;
    this.horizontalOffset = horizontalOffset;
    if(horizontalOffset != horizontal.getValue())
      updateScrollBars();
    textAreaPainter.repaint();
  }

  public final String getSelectedText()
  {
    if(selectionStart == selectionEnd)
      return null;

    if(rectSelect)
    {

      Element map = document.getDefaultRootElement();

      int start = selectionStart - map.getElement(selectionStartLine)
	.getStartOffset();
      int end = selectionEnd - map.getElement(selectionEndLine)
	.getStartOffset();

      if(end < start)
      {
	int tmp = end;
	end = start;
	start = tmp;
      }

      StringBuffer buf = new StringBuffer();
      Segment seg = new Segment();

      for(int i = selectionStartLine; i <= selectionEndLine; i++)
      {
	Element lineElement = map.getElement(i);
	int lineStart = lineElement.getStartOffset();
	int lineEnd = lineElement.getEndOffset() - 1;
	int lineLen = lineEnd - lineStart;

	lineStart = Math.min(lineStart + start,lineEnd);
	lineLen = Math.min(end - start,lineEnd - lineStart);

	getText(lineStart,lineLen,seg);
	buf.append(seg.array,seg.offset,seg.count);

	if(i != selectionEndLine)
	  buf.append('\n');
      }

      return buf.toString();
    }
    else
    {
      return getText(selectionStart,
		     selectionEnd - selectionStart);
    }
  }

  // TextArea  Layout 

  private static String CENTER = "center";
  private static String RIGHT = "right";
  private static String LEFT = "left";
  private static String BOTTOM = "bottom";

  class ScrollLayout implements LayoutManager
  {
    Component center;
    Component left;
    Component right;
    Component bottom;

    public void addLayoutComponent(String name, Component comp)
    {
      if(name.equals(CENTER))
	center = comp;
      else if(name.equals(RIGHT))
	right = comp;
      else if(name.equals(LEFT))
	left = comp;
      else if(name.equals(BOTTOM))
	bottom = comp;
    }

    public void removeLayoutComponent(Component comp)
    {
      if(center == comp)
	center = null;
      else if(right == comp)
	right = null;
      else if(left == comp)
	left = null;
      else if(bottom == comp)
	bottom = null;
    }

    public Dimension preferredLayoutSize(Container parent)
    {
      Dimension dim = new Dimension();
      Insets insets = getInsets();
      dim.width = insets.left + insets.right;
      dim.height = insets.top + insets.bottom;

      Dimension leftPref = left.getPreferredSize();
      dim.width += leftPref.width;
      Dimension centerPref = center.getPreferredSize();
      dim.width += centerPref.width;
      dim.height += centerPref.height;
      Dimension rightPref = right.getPreferredSize();
      dim.width += rightPref.width;
      Dimension bottomPref = bottom.getPreferredSize();
      dim.height += bottomPref.height;

      return dim;
    }

    public Dimension minimumLayoutSize(Container parent)
    {
      Dimension dim = new Dimension();
      Insets insets = getInsets();
      dim.width = insets.left + insets.right;
      dim.height = insets.top + insets.bottom;

      Dimension leftPref = left.getMinimumSize();
      dim.width += leftPref.width;
      Dimension centerPref = center.getMinimumSize();
      dim.width += centerPref.width; 
      dim.height += centerPref.height;
      Dimension rightPref = right.getMinimumSize();
      dim.width += rightPref.width;
      Dimension bottomPref = bottom.getMinimumSize();
      dim.height += bottomPref.height;

      return dim;
    }

    public void layoutContainer(Container parent)
    {
      Dimension size = parent.getSize();
      Insets insets = parent.getInsets();
      int itop = insets.top;
      int ileft = insets.left;
      int ibottom = insets.bottom;
      int iright = insets.right;

      int rightWidth = right.getPreferredSize().width;
      int leftWidth = left.getPreferredSize().width;
      int bottomHeight = bottom.getPreferredSize().height;
      int centerWidth = size.width - leftWidth - rightWidth -
	ileft - iright;
      int centerHeight = size.height - bottomHeight - itop -
	ibottom;

      left.setBounds(
	ileft,
	itop,
	leftWidth,
	centerHeight);

      center.setBounds(
	ileft + leftWidth,
	itop,
	centerWidth,
	centerHeight);

      right.setBounds(
	ileft + leftWidth + centerWidth,
	itop,
	rightWidth,
	centerHeight);

      bottom.setBounds(
	ileft,
	itop + centerHeight,
	size.width - rightWidth - ileft - iright,
	bottomHeight);
    }
  }

  class AdjustHandler implements AdjustmentListener
  {
    public void adjustmentValueChanged(final AdjustmentEvent evt)
    {
      if(!scrollBarsInitialized)
	return;

      SwingUtilities.invokeLater(new Runnable()
	{
	  public void run()
	  {
	    if(evt.getAdjustable() == vertical)
	      setFirstLine(vertical.getValue());
	    else
	      setHorizontalOffset(-horizontal.getValue());
	  }
	});
    }
  }

  class DocumentHandler implements DocumentListener
  {
    public void insertUpdate(DocumentEvent evt)
    {
      documentChanged(evt);

      if(!document.isLoaded())
	return;

      int offset = evt.getOffset();
      int length = evt.getLength();

      int newStart;
      int newEnd;

      boolean change = false;

      if(selectionStart > offset || (selectionStart 
				     == selectionEnd && selectionStart == offset))
      {
	change = true;
	newStart = selectionStart + length;
      }
      else
	newStart = selectionStart;

      if(selectionEnd >= offset)
      {
	change = true;
	newEnd = selectionEnd + length;
      }
      else
	newEnd = selectionEnd;

      if(change)
	select(newStart,newEnd);
    }

    public void removeUpdate(DocumentEvent evt)
    {
      documentChanged(evt);

      if(!document.isLoaded())
	return;

      int offset = evt.getOffset();
      int length = evt.getLength();

      int newStart;
      int newEnd;

      boolean change = false;

      if(selectionStart > offset)
      {
	change = true;

	if(selectionStart > offset + length)
	  newStart = selectionStart - length;
	else
	  newStart = offset;
      }
      else
	newStart = selectionStart;

      if(selectionEnd > offset)
      {
	change = true;

	if(selectionEnd > offset + length)
	  newEnd = selectionEnd - length;
	else
	  newEnd = offset;
      }
      else
	newEnd = selectionEnd;

      if(change)
	select(newStart,newEnd);
    }

    public void changedUpdate(DocumentEvent evt)
    {
    }
  }

  class MouseHandler extends MouseAdapter
  {
    public void mousePressed(MouseEvent evt)
    {
      requestFocus();

      focusedComponent = EditTextArea.this;
      setCaretVisible(true);

      if((evt.getModifiers() & InputEvent.BUTTON3_MASK) != 0
	 && popup != null)
      {
	popup.show(textAreaPainter,evt.getX(),evt.getY());
	return;
      }

      dragStartLine = yToLine(evt.getY());
      dragStartOffset = xToOffset(dragStartLine,evt.getX());
      int dot = getLineStartOffset(dragStartLine) + dragStartOffset;

      clickCount = evt.getClickCount();
      switch(clickCount)
      {
      case 1:
	doSingleClick(evt,dot);
	break;
      case 2:
	try
	{
	  doDoubleClick(evt,dot);
	}
	catch(BadLocationException bl)
	{
	  bl.printStackTrace();
	}
	break;
      case 3:
	doTripleClick(evt);
	break;
      }
    }

    private void doSingleClick(MouseEvent evt, int dot)
    {
      if((evt.getModifiers() & InputEvent.SHIFT_MASK) != 0)
      {
	rectSelect = (evt.getModifiers() & InputEvent.CTRL_MASK) != 0;
	select(getMarkPosition(),dot);
      }
      else
	setCaretPosition(dot);
    }

    private void doDoubleClick(MouseEvent evt, int dot) throws BadLocationException
    {
      if(getLineLength(dragStartLine) == 0)
	return;

      try
      {
	int bracket = TextUtilities.findMatchingBracket(
	  document,Math.max(0,dot - 1));
	if(bracket != -1)
	{
	  int mark = getMarkPosition();
	  if(bracket > mark)
	  {
	    bracket++;
	    mark--;
	  }
	  select(mark,bracket);
	  return;
	}
      }
      catch(BadLocationException bl)
      {
	bl.printStackTrace();
      }

      String lineText = getLineText(dragStartLine);
      if(dragStartOffset == getLineLength(dragStartLine))
	dragStartOffset--;

      int wordStart = TextUtilities.findWordStart(lineText,
						  dragStartOffset,null);
      int wordEnd = TextUtilities.findWordEnd(lineText,
					      dragStartOffset+1,null);

      int lineStart = getLineStartOffset(dragStartLine);
      select(lineStart + wordStart,lineStart + wordEnd);
    }

    private void doTripleClick(MouseEvent evt)
    {
      select(getLineStartOffset(dragStartLine),
	     getLineEndOffset(dragStartLine)-1);
    }
  }

  class ComponentHandler extends ComponentAdapter
  {
    public void componentResized(ComponentEvent evt)
    {
      recalculateVisibleLines();
      scrollBarsInitialized = true;
    }
  }

  class DragHandler implements MouseMotionListener
  {
    public void mouseDragged(MouseEvent evt)
    {
      if(popup != null && popup.isVisible())
	return;

      setSelectionRectangular((evt.getModifiers() & InputEvent.CTRL_MASK) != 0);

      switch(clickCount)
      {
      case 1:
	doSingleDrag(evt);
	break;
      case 2:
	doDoubleDrag(evt);
	break;
      case 3:
	doTripleDrag(evt);
	break;
      }
    }

    public void mouseMoved(MouseEvent evt) {}

    private void doSingleDrag(MouseEvent evt)
    {
      select(getMarkPosition(),xyToOffset(
	evt.getX(),evt.getY()));
    }

    private void doDoubleDrag(MouseEvent evt)
    {
      int markLineStart = getLineStartOffset(dragStartLine);
      int markLineLength = getLineLength(dragStartLine);
      int mark = dragStartOffset;

      int line = yToLine(evt.getY());
      int lineStart = getLineStartOffset(line);
      int lineLength = getLineLength(line);
      int offset = xToOffset(line,evt.getX());

      String lineText = getLineText(line);
      String markLineText = getLineText(dragStartLine);

      if(markLineStart + dragStartOffset > lineStart + offset)
      {
	if(offset != 0 && offset != lineLength)
	{
	  offset = TextUtilities.findWordStart(
	    lineText,offset,null);
	}

	if(markLineLength != 0)
	{
	  mark = TextUtilities.findWordEnd(
	    markLineText,mark,null);
	}
      }
      else
      {
	if(offset != 0 && lineLength != 0)
	{
	  offset = TextUtilities.findWordEnd(
	    lineText,offset,null);
	}

	if(mark != 0 && mark != markLineLength)
	{
	  mark = TextUtilities.findWordStart(
	    markLineText,mark,null);
	}
      }

      select(markLineStart + mark,lineStart + offset);
    }

    private void doTripleDrag(MouseEvent evt)
    {
      int mark = getMarkLine();
      int mouse = yToLine(evt.getY());
      int offset = xToOffset(mouse,evt.getX());
      if(mark > mouse)
      {
	mark = getLineEndOffset(mark) - 1;
	if(offset == getLineLength(mouse))
	  mouse = getLineEndOffset(mouse) - 1;
	else
	  mouse = getLineStartOffset(mouse);
      }
      else
      {
	mark = getLineStartOffset(mark);
	if(offset == 0)
	  mouse = getLineStartOffset(mouse);
	else
	  mouse = getLineEndOffset(mouse) - 1;
      }
      select(mark,mouse);
    }
  }

  class FocusHandler implements FocusListener
  {
    public void focusGained(FocusEvent evt)
    {
      setCaretVisible(true);
      focusedComponent = EditTextArea.this;
    }

    public void focusLost(FocusEvent evt)
    {
      setCaretVisible(false);
    }
  }

  public final void blinkCaret()
  {
    blink = !blink;
    textAreaPainter.invalidateSelectedLines();
  }

  static class CaretBlinker implements ActionListener
  {
    public void actionPerformed(ActionEvent evt)
    {
      if(focusedComponent != null	&& focusedComponent.hasFocus())
	focusedComponent.blinkCaret();
    }
  }

  static
  {
    caretTimer = new Timer(250,new CaretBlinker());
    caretTimer.setInitialDelay(250);
    caretTimer.start();
  }
}
