package kudos.ide.io;

import java.io.*;
import javax.swing.text.*;

import kudos.ide.form.*;
import kudos.ide.buffer.*;
import kudos.ide.tools.*;

public class IORequest implements Runnable {
  private int type;

  public static final int LOAD = 0;
  public static final int SAVE = 1;

  private String path;
  private String markersPath;

  private MainView mainView;
  private Buffer buffer;
  private VirtualFileSystem vfs;

  public IORequest(int type, MainView mainView, Buffer buffer, 
		   String path, VirtualFileSystem vfs)
  {
    this.type = type;
    this.mainView = mainView;
    this.buffer = buffer;
    this.path = path;
    this.vfs = vfs;

    markersPath = Util.getFileParent(path)
      + '.' + Util.getFileName(path)
      + ".marks";
  }

  public void run()
  {
    switch(type) {
      case LOAD:
	load();
	break;
      case SAVE:
	save();
	break;
    }
  }

  private void load()
  {
    InputStream in = null;
    try {
      in = vfs._createInputStream(mainView,buffer,path,false);
      buffer = buffer._read(in);
      setBuffer();
    } catch(BadLocationException bl) {
    } catch(IOException io) {
    }
  }

  private void save()
  {
    OutputStream out = null;
    try {
      out = vfs._createOutputStream(mainView,buffer,path);
      buffer._write(out);
    } catch(BadLocationException bl) {
    } catch(IOException io) {
    }
  }

  public void setBuffer() {
    mainView.setBuffer(buffer);
    mainView.getEditTextArea().setCaretPosition(
      mainView.getEditTextArea().getDocumentCaret());
    mainView.getEditTextArea().getTextAreaPainter().repaint();
  }

  public String toString() {
    return getClass().getName() + 
      "[type=" + (type == LOAD ? "LOAD" : "SAVE") + 
      ",view=" + mainView + ",buffer=" + buffer + ",vfs=" + vfs + "]";
  }
}
