package kudos.ide.io;

import javax.swing.*;

public class WorkThread extends Thread {
  private int requestCount;
  private int awtRequestCount;

  private Request firstRequest = null;
  private Request lastRequest = null;

  private Object lock = new Object();

  public WorkThread(String name) {
    super(name);
    setDaemon(true);
    setPriority(4);
  }

  public void addWorkRequest(Runnable run, boolean inAWT) {
    if((inAWT && awtRequestCount == 0) &&
       (SwingUtilities.isEventDispatchThread()))
    {
      run.run();
      return;
    }

    Request request = new Request(run,inAWT);

    synchronized (lock) {
      if(firstRequest == null && lastRequest == null) {
	firstRequest = lastRequest = request;
      }
      else {
	lastRequest.next = request;
	lastRequest = request;
      }

      requestCount++;

      lock.notify();
    }
  }

  public void run() {
    for(;;) {
      doRequests();
    }
  }

  private void doRequest(final Request request) {
    try {
      request.run.run();
    }
    catch(Throwable t){}
  }

  private void doRequests() {
    while(firstRequest != null) {
      doRequest(getNextRequest());
    }

    synchronized (lock) {
      lock.notifyAll();

      try {
	lock.wait();
      } catch(InterruptedException ie) {
      }
    }
  }

  private Request getNextRequest() {
    synchronized (lock) {
      Request request = firstRequest;
      firstRequest = firstRequest.next;
      if(firstRequest == null)
	lastRequest = null;

      request.alreadyRun = true;

      StringBuffer buf = new StringBuffer("request queue is now: ");
      Request _request = request.next;
      while (_request != null) {
	buf.append(_request.id);
	if(_request.next != null)
	  buf.append(",");
	_request = _request.next;
      }

      return request;
    }
  }

  static int ID;

  static class Request {
    int id = ++ID;

    Runnable run;
    boolean inAWT;

    boolean alreadyRun;

    Request next;

    Request(Runnable run, boolean inAWT) {
      this.run = run;
      this.inAWT = inAWT;
    }

    public String toString() {
      return "[id=" + id + ",run=" + run + "]";
    }
  }
}
