package kudos.ide.listener;

import java.io.*;
import javax.swing.text.html.parser.Parser;
import kudos.ide.form.*;
import kudos.ide.tools.Tools;
import kudos.schemej.*;
import kudos.schemej.util.*;
import kudos.schemej.stdlib.*;
import kudos.lang.*;
import kudos.exns.*;

public class Listener {
  private static MainView mainView = null;

  private int leftpar = 0;
  private int rightpar = 0;
  private static int message;

  public static int CANCEL = 0;
  public static int START = 1;
  public static int BYE = 2;
  public static int TERMINATION = 3;
  public static int END = 4;
  public static int ERROR = 9;

  private boolean isPrompt = false;
  private boolean isCommand = false;
  private static boolean isEnter = false;

  private String prompt = null;
  private StringBuffer buffer = null;
  private static String list = null;
  private static CharArrayReader inp = null;


  // for SchemeJ interpretation
  private kudos.schemej.Parser parser;     
  Evaluator eval;
  Environment env;

  
  private static String wMsg = Tools.getInterpreterStart();
  private static String gMsg = Tools.getInterpreterEnd();

  public Listener(MainView mainView) {
    this.mainView = mainView;
    eval = new Evaluator();
    env = new SchemeJEnvironment(null);
    
    setPrompt(true);
  }

  public void repl() {

    if(message == START) {
      sayWelcome();
    }
    else if(message == BYE) {
      sayGoodbye();
    }
    else if(message == TERMINATION) {
      ;
    }
    else if(message == END) {
      sayEnd();
    }
    else if(message == ERROR) {
      sayError();
    }
    else if(message == CANCEL) {
    }

    sayPrompt();
    message = 0;
  }

  public void setMessage(int message) {
    this.message = message;
  }

  public void isEnter() {
    isEnter = true;
  }

  private void setPrompt(boolean isPrompt) {
    this.isPrompt = isPrompt;
    if(isPrompt)
      prompt = "=> ";
    else
      prompt = " > ";
  }

  private static void sayWelcome() {
    mainView.getInterpreterView().setTextInput(wMsg + "\n");
  }

  private static void sayGoodbye() {
    mainView.getInterpreterView().setTextInput(gMsg + "\n");
  }

  private static void sayError() {
    mainView.getInterpreterView().setTextInput("Error: unbalanced " +
					       "parentheses." + "\n");
  }

  private static void sayEnd() {
    mainView.getInterpreterView().setTextInput("Initialized." + "\n");
  }

  public void say(String text) {
    mainView.getInterpreterView().setTextInput(text);
  }

  private static void sayTermination(String text) {
    if (isEnter) {
      mainView.getInterpreterView().setTextInput("\n");
      isEnter = false;
    }
    mainView.getInterpreterView().setTextInput(text + "\n");
  }

  private void sayPrompt() {
    mainView.getInterpreterView().setTextInput(prompt);
  }

  public void command(String text) {
    if (!isEnter) {
      if (text.equals("(clear)") || text.equals("(exit)")) {
	if (text.equals("(clear)")) {
	  if (isPrompt) {
	    message = START;
	    mainView.getInterpreterView().setText("");
	  }
	}

	if (text.equals("(exit)")) {
	  if (!isPrompt) {
	    leftpar = 0;
	    rightpar = 0;
	    buffer = null;
	    message = END;
	    setPrompt(true);
	  }
	}
	isCommand = true;
      }
    }
  }

  public void readCharecter(String text) {
    command(text);
		
    if (isCommand) {
      isCommand = false;
      return;
    }

    list = text;

    if (buffer == null) {
      buffer = new StringBuffer();
    }

    char[] clist = list.toCharArray();
    boolean commentary = true;

    for (int i = 0; i < list.length();i++) {
      if(clist[i] == ';')
	commentary = false;

      if (commentary) {
	if (clist[i] == '(') {
	  leftpar++;
	}
	else if (clist[i] == ')') {
	  rightpar++;
	}

	if ((buffer.length() != 0) && 
	    (buffer.charAt(buffer.length() - 1) == ' ') && 
	    (clist[i] == ' ')) 
	{
	}
	else if ((clist[i] != '\n') && (clist[i] != '\t')) {
	  buffer.append(clist[i]);
	}
	else if (clist[i] == '\n') {
	  buffer.append(' ');
	}
      }

      if (clist[i] == '\n')
	commentary = true;
    }

    if ((leftpar == rightpar) && (buffer.length() != 0) || isEnter) {
      leftpar = 0;
      rightpar = 0;
      list = buffer.toString();
      buffer = null;
      setPrompt(true);
      charArrayTransmission(list);
      return;
    }
    
    if (!isEnter) {
      if(leftpar > rightpar) {
	if(!(buffer.length() != 0 && 
	     buffer.charAt(buffer.length() - 1) == ' '))
	{
	  buffer.append(' ');
	}
	setPrompt(false);
	return;
      }
      else if (leftpar < rightpar) {
	leftpar = 0;
	rightpar = 0;
	buffer = null;
	message = ERROR;
	setPrompt(true);
      }
      else if (buffer.toString().equals("")) {
	leftpar = 0;
	rightpar = 0;
	buffer = null;
      }
    }
  }

  private void charArrayTransmission(String text) {
    char[] clist = text.toCharArray();
    inp = new CharArrayReader(clist);

    try {
      parser = new kudos.schemej.Parser(new Lexer(inp));
      java_cup.runtime.Symbol sym = parser.parse();
      SExpr sexpr = (SExpr) sym.value;

      Expr expr = (Expr) sexpr.accept(new SExprParser());
      Object value = expr.eval(eval, env,
			       new PrintContinuation(env, null));
      message = TERMINATION;

      if (value instanceof SchemeJException) {
	sayTermination(((SchemeJException) value).getMessage());
      }
      else {
	sayTermination(value.toString());
      }
    } catch (Exception e) { 
    }
  }
}
