/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * KudosClassLoader.java:
 *   - created by Youngwoo Baek
 *   - modified by Cheoljoo Jeong
 *
 * Copyright(c) 2000. ETRI-CSTL. All rights reserved.
 *
 * $Log$
 *
 */

package kudos.jvm;

import java.util.*;
import java.io.*;
import javax.swing.tree.*;
import javax.swing.JTree;
import kudos.ide.form.KudosTreeObserver;
import kudos.ide.form.JVMBrowserView;

/**
 * KudosClassLoader is the class loader which is used by the
 * SchemeJ intepreter for its support for Java classes.
 * KudosClassLoader is a singleton class.
 */
public class KudosClassLoader extends ClassLoader {
  private static KudosClassLoader instance = null;

  /* used for maintaining loaded classes */
  private Hashtable loadedAllClasses = new Hashtable();
  private Hashtable loadedSystemClasses = new Hashtable();
  private Hashtable loadedUserClasses = new Hashtable();
  private Hashtable classHashtable = new Hashtable();
  private Hashtable instanceHashtable = new Hashtable();
 
  /* used for serving kudos.ide classes */
  private DefaultTreeModel systemClassTreeModel;
  private DefaultTreeModel userClassTreeModel;
  private DefaultMutableTreeNode top;
  
  private boolean isInserted;
  private boolean isEntrance;
  private int callcnt;
  private int totalChildCount;
  private int currentlyChildCount;
  private int traverseCount;

  private KudosTreeObserver observer = null;


  /**
   * constructor
   */
  public KudosClassLoader(KudosTreeObserver observer) {
    initialize();
    this.observer = observer;
  }

  /**
   * constructor
   */
  public KudosClassLoader() {
    initialize();
    //    this.observer = JVMBrowserView.getInstance();
  }	

  /**
   * Returns the sole instance of this class.
   */
  public static KudosClassLoader getInstance() {
    if (instance == null) {
      instance = new KudosClassLoader();
    }
    return instance;
  }

  public static KudosClassLoader getInstance(KudosTreeObserver observer) {
    if (instance == null) {
      instance = new KudosClassLoader(observer);
    }
    return instance;
  }

  /**
   * Returns the Class object specified by the classname.
   * In fact, the real work is done by the 'loadClass' method.
   */
  public Class getClass(String classname) throws ClassNotFoundException {
    try {
      Class theClass = null;
      
      if (loadedAllClasses.containsKey(classname)) {
	return (Class) loadedAllClasses.get(classname);
      }
      else {
	return loadClass(classname, false);
      }
    } catch (ClassNotFoundException e) {
      throw e;
    }
  }

  /**
   * Returns the Class object specified by the classname.
   * When 'resolve' is true, subobject resolution is carried out.
   */
  protected Class loadClass(String classname, boolean resolve) 
    throws ClassNotFoundException 
  {
    Class theClass = null;

    if (classname.startsWith("java")) {
      try {
	// findSystemClass finds a class with the specified name,
	// loading it if necessary
	theClass = findSystemClass(classname);				
      } catch(ClassNotFoundException e) {
	throw e;
      }
    }
    // search for user-defined classes
    else {
      theClass = findUserDefinedClass(classname);
    }
		
    // if the class has been found
    if (theClass != null) {
      loadedAllClasses.put(classname, theClass);
      classHashtable.put(classname, instanceHashtable);
			
      // if the class is a system class
      if (classname.startsWith("java")) {				
	loadedSystemClasses.put(classname, theClass);
	if (observer != null) {
	  observer.systemClassChanged();
	}
      }
      // else if the class is user-defined class
      else {
	loadedUserClasses.put(classname, theClass);
	if (observer != null) {
	  observer.userDefineClassChanged();
	}
      }
    }
    else {
      throw new ClassNotFoundException();
    }
		
    if (resolve == true) {
      resolveClass((Class) loadedAllClasses.get(classname));		
    }
		
    return theClass;
		
  }
	
  /**
   * Finds a class with the specified name and returns the class.
   */
  private Class findUserDefinedClass(String classname) 
    throws ClassNotFoundException
  {		
    String filename;
		
    /*    
    if (classname.indexOf(".") >= 0) {
      filename = classname.substring(classname.lastIndexOf(".") + 1);
    }
    else {
      filename = classname;
      }
    */
    filename = classname;
		
    //filename = filename.replace('.', File.pathSeparatorChar);
    filename = filename.replace('.', '\\');
    filename += ".class";
    File loadingfile = new File(filename);
    System.out.println("filename=>" + filename);
    System.out.println("abs path=>" + loadingfile.getAbsolutePath());
	
		
    if (!loadingfile.exists()) {
      System.out.println("No such file : " + loadingfile);
    }
    else if (!loadingfile.isFile()) {
      System.out.println("filename : " + loadingfile + " is not file");
    }
    else if (!loadingfile.canRead()) {
      System.out.println("filename : " + loadingfile + " can't read");
    }
		
    try {
      // read the bytes codes
      InputStream is = new FileInputStream(loadingfile);
      int bufsize = (int)loadingfile.length();
      byte buf[] = new byte[bufsize];
      is.read(buf, 0, bufsize);
      is.close();
			
      // define the class from the byte code 
      return defineClass(classname, buf, 0, buf.length);
    } catch (Exception e) {
      throw new ClassNotFoundException(classname);
    }
  } 

  /**
   * Returns the array of all the loaded classes.
   */
  public Class[] getAllLoadedClasses()
  {
    if (loadedAllClasses.isEmpty()) {
      return new Class[0];
    }
		
    Enumeration e = loadedAllClasses.elements();
    Class[] classes = new Class[loadedAllClasses.size()];
		
    int i = 0;
    while (e.hasMoreElements()) {
      classes[i++] = (Class) e.nextElement();
    }
		
    return classes;		
  }
	
  /**
   * Returns the array of all the lodaed system classes.
   */
  public Class[] getLoadedSystemClasses() {
    if (loadedSystemClasses.isEmpty()) {
      return new Class[0];
    }
		
    Enumeration e = loadedSystemClasses.elements();
    Class[] classes = new Class[loadedSystemClasses.size()];
		
    int i = 0;
    while (e.hasMoreElements()) {
      classes[i++] = (Class) e.nextElement();
    }
		
    return classes;
  }
	
  /**
   * Returns the array of all the loaded user-defined classes.
   */
  public Class[] getLoadedUserClasses() {
    if (loadedUserClasses.isEmpty()) {
      return new Class[0];
    }
		
    Enumeration e = loadedUserClasses.elements();
    Class[] classes = new Class[loadedUserClasses.size()];
		
    int i = 0;
    while (e.hasMoreElements()) {
      classes[i++] = (Class) e.nextElement();
    }
		
    return classes;
  } 

  /**
   * Overriding the findclass method of ClassLoader.
   */
  public Class findClass(String classname) throws ClassNotFoundException {
    Class theClass = null;

    try {
      if (loadedAllClasses.containsKey(classname)) {
	theClass = (Class) loadedAllClasses.get(classname);
      }		
      // if not found, consult the parent class loader
      if (theClass == null) {
	theClass = super.findClass(classname);
      }
      return theClass;
    } catch (ClassNotFoundException e) {
      throw e;
    }
  } 

	
  /**
   * Make a GUI tree.
   */
  private void makeTree(DefaultMutableTreeNode top, 
			TreeModel systemClassTreeModel, 
			Class insertClass)
  {
    // if only root node exists
    if (callcnt == 0) {
      callcnt++;
      DefaultMutableTreeNode insertNode = 
	new DefaultMutableTreeNode(insertClass);
      DefaultMutableTreeNode rootNode = 
	(DefaultMutableTreeNode)systemClassTreeModel.getRoot();
      rootNode.add(insertNode);
			
      return;
    }

    currentlyChildCount = 0;
    isEntrance = false;
    travserNodes(top, insertClass);
  } 

  /**
   * Traverse nodes.
   */
  private void travserNodes(TreeNode parentNode, Class insertClass)
  {
    Enumeration enum = parentNode.children();
    for (int index=0; enum.hasMoreElements(); index++)
    {
      DefaultMutableTreeNode childNode = 
	(DefaultMutableTreeNode) enum.nextElement();
      currentlyChildCount++;

      if (insertClassIntoTreeModel(childNode, insertClass))
      {
	break;
      }

      if ( !childNode.isLeaf())
      {
        travserNodes(childNode, insertClass);
      }
    } // end of for-loop

    totalChildCount = 0; //setting
    if ( (currentlyChildCount == getTotalChildCount(top)) && 
	 !isInserted && !isEntrance)
    {
      // insert insertClass into Root's childNode
      isEntrance = true;
      DefaultMutableTreeNode insertNode = new DefaultMutableTreeNode(insertClass);
      DefaultMutableTreeNode rootNode = 
	(DefaultMutableTreeNode) systemClassTreeModel.getRoot();
      rootNode.add(insertNode);

      isInserted = true;
    }
  } // method of "travserNodes()"

  private boolean insertClassIntoTreeModel(DefaultMutableTreeNode node, 
					   Class insertClass)
  {
    boolean isInsertedBetweenNodes = false;
    boolean isInsertedIntoClass = false;
    isInserted = false;

    Class theClass = (Class)node.getUserObject();
    Class tmpClass = theClass;

    while (tmpClass.getSuperclass() != null)
    {
      //System.out.println( (tmpClass.getSuperclass()).getName() );
      if (insertClass.getName() == (tmpClass.getSuperclass()).getName())
      {
				// insert between nodes
        DefaultMutableTreeNode insertNode = 
	  new DefaultMutableTreeNode(insertClass);
	((DefaultMutableTreeNode) (node.getParent())).add(insertNode);
	insertNode.add(node);
										
	isInsertedBetweenNodes = true;
	break;
      }
      else
      {
	tmpClass = tmpClass.getSuperclass();
      }
    }
		
    if (!isInsertedBetweenNodes)
    {
      Class superClass = insertClass.getSuperclass();
      while(superClass != null)
      {
        //System.out.println( superClass.getName() );
	if (theClass.getName() == superClass.getName())
	{
	  // insert insertClass into theClass
	  DefaultMutableTreeNode insertNode = 
	    new DefaultMutableTreeNode(insertClass);
	  node.add(insertNode);

	  isInsertedIntoClass = true;
	  break;
	}
	else
	{
	  superClass = superClass.getSuperclass();
	}
      }
    }

    if (!isInserted)
    {
      isInserted = isInsertedBetweenNodes | isInsertedIntoClass;
    }

    return isInserted;			
  } // method of "insertClassIntoTreeModel()"
		
  public TreeModel getLoadedSystemClassHierarchy()
  {
    initialize();
		
    Enumeration e = loadedSystemClasses.elements();
    Class[] systemClasses = new Class[loadedSystemClasses.size()];
		
    int i=0;
    while(e.hasMoreElements())
    {
      systemClasses[i++] = (Class)e.nextElement();
    }
		
    for (int index=0; index<systemClasses.length; index++)
    {			
      Class loadedClass = systemClasses[index];
      makeTree(top, systemClassTreeModel, loadedClass);
    }

    return systemClassTreeModel;
  } // end of method "getLoadedSystemClassHierarchy()
	
  public TreeModel getLoadedUserClassHierarchy()
  {
    initialize();
		
    Enumeration e = loadedUserClasses.elements();
    Class[] userClasses = new Class[loadedUserClasses.size()];
		
    int i=0;
    while(e.hasMoreElements())
    {
      userClasses[i++] = (Class)e.nextElement();
    }
		
    for (int index=0; index<userClasses.length; index++)
    {			
      Class loadedClass = userClasses[index];			
      makeTree(top, systemClassTreeModel, loadedClass);
    }

    return userClassTreeModel;
  } // end of method "getLoadedUserClassHierarchy()


  private int getTotalChildCount(TreeNode parentNode)
  {
    Enumeration enum = parentNode.children();

    for (int index=0; enum.hasMoreElements(); index++)
    {
      TreeNode childNode = (TreeNode)enum.nextElement();
      totalChildCount++;

      if (!childNode.isLeaf())
      {
	getTotalChildCount(childNode);
      }
    }

    return totalChildCount;
  }	
	
  private void initialize() {
    try {
      // now
      systemClassTreeModel = null;
      userClassTreeModel = null;
      top = null;
      Class theClass = Class.forName("java.lang.Object");
      
      top = new DefaultMutableTreeNode(theClass);    
      systemClassTreeModel = new DefaultTreeModel(top);       
      userClassTreeModel = systemClassTreeModel;
      
      isInserted = false;
      isEntrance = false;
      callcnt = 0;
      totalChildCount = 0;
      currentlyChildCount = 0;
      traverseCount = 0;
    } catch (ClassNotFoundException e) { 
    }
  } // end of method "initilize()"

  public Class[] getSubclasses(String classname)
  {
    //Class theClass = null;
    DefaultMutableTreeNode targetNode = null;

    if (!loadedAllClasses.containsKey(classname))
    {
      System.out.println("Class :" + classname + " is not loaded in the Java Virtual Machine!!!");
      return new Class[0];
    }

    if (classname.startsWith("java")) // if class is "systemClass"
    {
      TreeModel systemClassTreeModel = getLoadedSystemClassHierarchy();
      DefaultMutableTreeNode top = 
	(DefaultMutableTreeNode) systemClassTreeModel.getRoot();
      targetNode = findTargetNode(classname, top);
    }
    else // if class is "user-defined class"
    {
      TreeModel userClassTreeModel = getLoadedUserClassHierarchy();
      DefaultMutableTreeNode top = 
	(DefaultMutableTreeNode) userClassTreeModel.getRoot();

      targetNode = findTargetNode(classname, top);
    }

    Hashtable subClassHashtable = new Hashtable();
    makeSubclassHashtable(subClassHashtable, targetNode);

    Enumeration enum = subClassHashtable.elements();
    Class[] subClasses = new Class[subClassHashtable.size()];
    int i=0;
    while(enum.hasMoreElements())
    {
      subClasses[i++] = (Class)enum.nextElement();
    }

    return subClasses;

  } // end of method "getSubclasses()"

  private DefaultMutableTreeNode 
    findTargetNode(String classname, 
		   DefaultMutableTreeNode parentNode)
  {
    if (parentNode == null)
    {
      return null;
    }

    if ( (((Class)(parentNode.getUserObject())).getName()).equals(classname) )
    {
      return parentNode;
    }

    DefaultMutableTreeNode tmpNode = null;
    int childCount = parentNode.getChildCount();
    for (int index=0; index<childCount; index++)
    {
      if (tmpNode == null)
      {
        tmpNode = 
	  findTargetNode(classname, 
			 (DefaultMutableTreeNode)parentNode.getChildAt(index));
      }
    }

    return tmpNode;
  }

  /* 
  public void printNodes(TreeNode parentNode)
  {
    Enumeration enum = parentNode.children();
    
    for (int index=0; enum.hasMoreElements(); index++)
      {
	TreeNode childNode = (TreeNode)enum.nextElement();
	System.out.println( parentNode + "-" + index + " : " + childNode );
	if ( !childNode.isLeaf())
	  {
	    printNodes(childNode);
	  }
      }
  }
  */

  private void makeSubclassHashtable(Hashtable subClassHashtable, 
				     TreeNode parentNode)
  {
    Enumeration enum = parentNode.children();

    for (int index=0; enum.hasMoreElements(); index++)
    {
      DefaultMutableTreeNode childNode = 
	(DefaultMutableTreeNode) enum.nextElement();
      subClassHashtable.put(((Class)(childNode.getUserObject())).getName(), 
			    ((Class)(childNode.getUserObject())));
      if (!childNode.isLeaf())
      {
	makeSubclassHashtable(subClassHashtable, childNode);
      }
    }
  }


  public Class[] getDirectSubclasses(String classname)
  {
    //Class theClass = null;
    DefaultMutableTreeNode targetNode = null;

    if (!loadedAllClasses.containsKey(classname))
    {
      System.out.println("Class :" + classname + 
			 " is not loaded in the Java Virtual Machine!!!");
      return new Class[0];
    }

    if (classname.startsWith("java")) // if class is "systemClass"
    {
      TreeModel systemClassTreeModel = getLoadedSystemClassHierarchy();
      DefaultMutableTreeNode top = 
	(DefaultMutableTreeNode) systemClassTreeModel.getRoot();
      targetNode = findTargetNode(classname, top);
    }
    else // if class is "user-defined class"
    {
      TreeModel userClassTreeModel = getLoadedUserClassHierarchy();
      DefaultMutableTreeNode top = 
	(DefaultMutableTreeNode) userClassTreeModel.getRoot();
      targetNode = findTargetNode(classname, top);
    }

    Hashtable directSubClassHashtable = new Hashtable();
    makeDirectSubclassHashtable(directSubClassHashtable, targetNode);

    Enumeration enum = directSubClassHashtable.elements();
    Class[] subClasses = new Class[directSubClassHashtable.size()];
    int i=0;
    while(enum.hasMoreElements())
    {
      subClasses[i++] = (Class)enum.nextElement();
    }

    return subClasses;
  } // end of method "getDirectSubclasses()"

  private void makeDirectSubclassHashtable(Hashtable directSubClassHashtable, 
					   TreeNode parentNode)
  {
    Enumeration enum = parentNode.children();

    for (int index=0; enum.hasMoreElements(); index++)
    {
      DefaultMutableTreeNode childNode = 
	(DefaultMutableTreeNode) enum.nextElement();
      directSubClassHashtable.put(((Class)(childNode.getUserObject())).getName(), 
				  ((Class)(childNode.getUserObject())) );
    }
  }


  public Class[] getSuperclasses(String classname)
  {
    DefaultMutableTreeNode targetNode = null;

    if (!loadedAllClasses.containsKey(classname))
    {
      System.out.println("Class :" + classname + 
			 " is not loaded in the Java Virtual Machine!!!");
      return new Class[0];
    }

    if (classname.startsWith("java")) // if class is "systemClass"
    {
      TreeModel systemClassTreeModel = getLoadedSystemClassHierarchy();
      DefaultMutableTreeNode top = 
	(DefaultMutableTreeNode) systemClassTreeModel.getRoot();
      targetNode = findTargetNode(classname, top);
    }
    else // if class is "user-defined class"
    {
      TreeModel userClassTreeModel = getLoadedUserClassHierarchy();
      DefaultMutableTreeNode top = 
	(DefaultMutableTreeNode) userClassTreeModel.getRoot();
      targetNode = findTargetNode(classname, top);
    }

    Hashtable superClassHashtable = new Hashtable();
    makeSuperclassHashtable(superClassHashtable, targetNode);

    Enumeration enum = superClassHashtable.elements();
    Class[] superClasses = new Class[superClassHashtable.size()];
    int i=0;
    while(enum.hasMoreElements())
    {
      superClasses[i++] = (Class)enum.nextElement();
    }

    return superClasses;
  } // end of method "getSuperclasses()"

  private void makeSuperclassHashtable(Hashtable SuperClassHashtable, TreeNode targetNode)
  {
    int index = 0;
    while ( !(((Class)(((DefaultMutableTreeNode)(targetNode.getParent())).getUserObject())).getName()).equals("java.lang.Object") )
    {
      SuperClassHashtable.put(new Integer(index), 
			      (Class)(((DefaultMutableTreeNode)targetNode.getParent()).getUserObject()) );
      targetNode = (DefaultMutableTreeNode) targetNode.getParent();
      index++;
    }
  } // end of makeSuperclassHashtable()


  public Class getDirectSuperclass(String classname)
  {
    DefaultMutableTreeNode targetNode = null;

    if (!loadedAllClasses.containsKey(classname))
    {
      System.out.println("Class :" + classname + 
			 " is not loaded in the Java Virtual Machine!!!");
      return null;
    }

    if (classname.startsWith("java")) // if class is "systemClass"
    {
      TreeModel systemClassTreeModel = getLoadedSystemClassHierarchy();
      DefaultMutableTreeNode top = 
	(DefaultMutableTreeNode) systemClassTreeModel.getRoot();
      targetNode = findTargetNode(classname, top);
    }
    else // if class is "user-defined class"
    {
      TreeModel userClassTreeModel = getLoadedUserClassHierarchy();
      DefaultMutableTreeNode top = 
	(DefaultMutableTreeNode) userClassTreeModel.getRoot();
      targetNode = findTargetNode(classname, top);
    }

    DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode)targetNode.getParent();

    return (Class)(parentNode.getUserObject());

  } // end of method "getDirectSuperclass()"


  public Object[] getInstances(String classname)
  {
    if (!classHashtable.containsKey(classname))
    {
      System.out.println("There is not class name" + " : " + 
			 classname + " in the Java Virtual Machine");
    }

    Hashtable instanceHashtable = (Hashtable)classHashtable.get(classname);
		
    Enumeration e = instanceHashtable.elements();
    Object[] objects = new Object[instanceHashtable.size()];
		
    int i=0;
    while(e.hasMoreElements()) {
      objects[i++] = e.nextElement();
    }
		
    return objects;
  }
	
  public void putInstance(String classname, String objectname, 
			  Object instantiatedObject)
  {
    Hashtable instanceHashtable = (Hashtable)classHashtable.get(classname);
			
    //int count = instanceHashtable.size();
    instanceHashtable.put(objectname, instantiatedObject);
  }
	
}
