/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * Closure.java: representation of the closure
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.lang;

import kudos.exns.*;

/**
 * When a procedure is defined through a lambda expression, 
 * a closure that includes the environment at the time of procedure
 * definition is created.
 *
 * @author Cheoljoo Jeong
 */

public class Closure extends Procedure {
  public String[] params;     // parameter names
  public Expr body;           // function body
  public Environment cloEnv;  // enclosing environment


  /**
   * constructor for anonymous procedures
   */
  public Closure(String[] params, Expr body, Environment parent) {
    this.params = new String[params.length];
    for (int i = 0; i < params.length; i++) {
      this.params[i] = params[i];
    }
    this.body = body;
    // in fact, env object sharing may result in a problem
    // through set! expressions.
    // so this env should be this.env = env.clone();
    // but this may be too burdensome!
    // devise an efficient env handling mechanism
    this.cloEnv = new Environment(parent);
  }

  /**
   * The `apply.'
   */
  public Object apply(ExprVisitorI evaluator, Object[] arguments, 
		      Environment r, Continuation k) 
    throws SchemeJException
  {
    try {
      for (int i = 0; i < arguments.length; i++) {
	cloEnv.extend(params[i], arguments[i]);
      }
      return body.eval(evaluator, cloEnv, k);
    } catch (SchemeJException e) {
      return k.resume(e);
    }
  }

  /**
   * Returns true if the given number is the same as the arity of the
   * procedure that the closure represents.
   */
  public boolean checkArity(int num) {
    if (num == params.length) {
      return true;
    }
    else {
      return false;
    }
  }
}
