/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * ControlStack.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.lang;

import kudos.exns.*;
import java.util.*;

/**
 * A control stack is a list of current frame, the previous
 * frame, its previous frame, and so on.
 * ControlStack class is a singleton class.
 *
 * @author Cheoljoo Jeong
 */

public class ControlStack {
  private static ControlStack instance = null;
  private Stack stack;

  /**
   * constructor
   */
  public ControlStack() {
    stack = new Stack();
  }

  /**
   * Returns the single instance of the class.
   */
  public static ControlStack getInstance() {
    if (instance == null) {
      instance = new ControlStack();
    }
    return instance;
  }

  /**
   * Add a call frame  to the control stack.
   */
  public void push(CallFrame frame) {
    stack.push(frame);
  }

  /**
   * Pops off the top frame from the stack and returns it.
   */
  public CallFrame pop() throws ControlStackEmptyException {
    try {
      Object o = stack.pop();
      return (CallFrame) o;
    } catch (EmptyStackException e) {
      throw new ControlStackEmptyException();
    }
  }

  /**
   * Returns the call frame at the top of the stack without popping
   * it off
   */
  public CallFrame top() throws ControlStackEmptyException {
    try {
      Object o = stack.peek();
      return (CallFrame) o;
    } catch (EmptyStackException e) {
      throw new ControlStackEmptyException();
    }
  }

  /**
   * Returns true iff the control stack is empty.
   */
  public boolean isEmpty() {
    return stack.empty();
  }
}




