/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * Environment.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.lang;

import java.util.*;
import kudos.exns.*;

/**
 * Environment is a set of bindings, where a binding is pair
 * of variable and its value.
 *
 * @author Cheoljoo Jeong
 */

public class Environment extends Hashtable { //implements ResumableI {
  //  private Binding[] table;
  
  public static Environment globalEnv = new Environment(null);
  public Environment prevEnv;

  /**
   * constructor
   */
  public Environment(Environment prevEnv) {
    //bindings = new Vector();
    this.prevEnv = prevEnv;
  }

  /**
   * Returns the base environment.
   */
  public static Environment getBaseEnv() {
    return globalEnv;
  }

  /**
   * Finds the binding for the given variable and returns the
   * associated value.
   */
  public Object lookup(String name) throws UnboundVarException {
    Binding binding = (Binding) this.get(name);
    Object value = null;
    // if not found in this env, lookup the parent environment
    if (binding == null) {
      if (prevEnv != null) {
	value = prevEnv.lookup(name);
      }
    }
    else {
      value = binding.getValue();
    }


    if (value != null) {
      return value;
    }
    else {
      throw new UnboundVarException("unbound var, " + name);
    }
  }

  /**
   * Entends the environment with the given binding
   */
  public void extend(Binding binding) {
    this.put(binding.getKey(), binding.getValue());
  }
    
  /**
   * Extends the environment with the given pair of name and value
   */
  public void extend(String name, Object value) {
    Binding binding = new Binding(name, value);
    this.put(name, binding);
  }

  /**
   * Updates the environment with the given pair of name and value
   */
  public void destructiveExtend(String name, Object value) {
    Binding binding = (Binding) this.get(name);

    if (binding == null) {
      extend(name, value);
    }
    else {
      binding.setValue(value);
    }
  }

  /**
   * Updates the environment with the given pair of name and value
   */
  public void update(String name, Object value) throws UnboundVarException {
    Binding binding = (Binding) this.get(name);

    if (binding == null) {
      throw new UnboundVarException("unbound var, " + name);
    }
    else {
      binding.setValue(value);
    }
  }

  /**
   * Prints the bindings in the environment.
   */
  public void printBindings() {
    try {
      Enumeration e = this.keys();
      while (e.hasMoreElements()) {
	String key = (String) e.nextElement();
	System.out.println("Key: " + key + ",  Value: " + lookup(key));
      }
    } catch (UnboundVarException e) {
    }
  }
}
