/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * ApplyContinuation.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej;

import kudos.lang.*;
import kudos.exns.*;

/**
 * A contination is `the operations remaining to be performed' 
 * and can be considered to be a control information explicitly
 * represented.
 *
 * @author Cheoljoo Jeong
 */

public class ApplyContinuation extends Continuation {
  private Expr[] args;

  /**
   * Constructor.
   * precondition: args are not null; 0-argument is passed as
   *               length-0 Expr array
   */
  public ApplyContinuation(Expr[] args, Environment r, Continuation k) 
  {
    super(r, k);

    this.args = new Expr[args.length];
    for (int i = 0; i < args.length; i++) {
      this.args[i] = args[i];
    }
  }

  /**
   * Resumes the apply continutation.
   * Receives a `Closure' object and applies it to the arguments.
   */
  public Object resume(Object v) {
    try {
      if (v instanceof Procedure) {
	// check the number of arguments
	if (!((Procedure) v).checkArity(args.length)) {
	  return k.resume(new WrongNumberOfRandsException("wrong number " +
							  "of args"));
	}
	else {
	  // call-by-value rule;
	  // arguments are evaluated in left-to-right order
	  Object[] randvals = new Object[args.length];
	  for (int i = 0; i < args.length; i++) {
	    randvals[i] = 
	      args[i].eval(Evaluator.evaluator, r,
			   new PrintErrorContinuation(r, null));
	  }
	  return ((Procedure) v).apply(Evaluator.evaluator, randvals, r, k);
	}
      }
      else {
	return k.resume(new InvalidRatorException("only procedures " +
						  "can be applied"));
      }
    } catch (SchemeJException e) {
      return k.resume(e);
    }
  }
}



