/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * IfContinuation.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej;

import kudos.lang.*;
import kudos.exns.*;

/**
 * A contination is `the operations remaining to be performed' 
 * and can be considered to be a control information explicitly
 * represented.
 *
 * @author Cheoljoo Jeong
 */

public class IfContinuation extends Continuation {
  private Expr thenExpr;
  private Expr elseExpr;

  /**
   * constructor
   */
  public IfContinuation(Expr thenExpr, Expr elseExpr, Environment r, 
			Continuation k) 
  {
    super(r, k);
    this.thenExpr = thenExpr;
    this.elseExpr = elseExpr;
  }

  /**
   * Resumes the if continutation.
   * Passed object v is the value of the test expression 
   * of the if expression.
   */
  public Object resume(Object v) {
    try {
      // if test has boolean type
      if (v instanceof Boolean) {
	// if test evaluates to true
	if (((Boolean) v).booleanValue() == true) {
	  return thenExpr.eval(Evaluator.evaluator, r, k);
	}
	else {
	  return elseExpr.eval(Evaluator.evaluator, r, k);
	}
      }
      else {
	return k.resume(new InvalidTypeException("boolean type required"));
      }
    } catch (SchemeJException e) {
      return k.resume(e);
    }
  }
}

