/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * PrintContinuation.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej;

import kudos.schemej.util.*;
import kudos.lang.*;
import kudos.exns.*;

/**
 * A print contination is the computation when the evaluation
 * of a form or an expression is finished.
 *
 * @author Cheoljoo Jeong
 */

public class PrintContinuation extends Continuation {

  /**
   * constructor
   */
  public PrintContinuation(Environment r, Continuation k) {
    super(r, k);
  }

  /**
   * Resumes the suspended computation.
   * Prints the result.
   */
  public Object resume(Object v) {
    String output = "";

    // in case of failure
    if (v instanceof SchemeJException) {
      switch (((SchemeJException) v).getErrorClass()) {
        case SchemeJException.SysError:
	  output = "Error: " + ((SchemeJException) v).getMessage();
	  break;
        case SchemeJException.EvalError:
	  output = "Error: " + ((SchemeJException) v).getMessage();
	  break;
        default:
	  output = "Error: " + ((SchemeJException) v).getMessage();
      }
      //((Exception) v).printStackTrace();
    }
    // in case of success
    else {
      //output = "Value: ";
      if (v instanceof Pair) {
	try {
	  output += (((Pair) v).accept(new SExprListPrinter()));
	} catch (SchemeJException e) {
	}
      }
      else if (v instanceof NullList) {
	output += "()";
      }
      else if (v instanceof SSymbol) {
	int id = ((SSymbol) v).getId();
	if (id == sym.SYMBOL) {
	  output += ((SSymbol) v).getName();
	}
	else if (id == sym.STRING) {
	  output = output + "\"" + ((SSymbol) v).getName() + "\"";
	}
	else if (id == sym.CHAR) {
	  output = output + "\\#" + ((SSymbol) v).getName();
	}
      }
      else if (v instanceof SNumber) {
	double doubleval = ((Double) ((SNumber) v).getValue()).doubleValue();
	int intval = ((Double) ((SNumber) v).getValue()).intValue();
	if ((doubleval - (double) intval) != 0) {
	  output += ((SNumber) v).getValue().toString();
	}
	else {
	  output += intval;
	}
      }
      else if (v instanceof String) {
	// String value can be returned from Java methods
	output = output + "\"" + v + "\"";
      }
      else if (v instanceof VarRefExpr) {
	output = output + ((VarRefExpr) v).name;
      }
      else if (v instanceof Closure) {
	output = output + "<Closure: " + v + ">";
      }
      else if (v instanceof Procedure) {
	output = output + "<Stdproc: " + v + ">";
      }
      else if (v == null) {
	output = output + "null";
      }
      else {
	output = output + v.toString();
      }
    }
    // print to console
    System.out.println(output);

    return new SchemeJException(output);
  }
}



