/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * SExprListPrinter.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej;

import kudos.schemej.util.*;
import kudos.exns.*;

/**
 * Pretty printer for SExpr objects, which prints s-expressions
 * in a linear list format.
 *
 * @author Cheoljoo Jeong
 */

public class SExprListPrinter implements SExprVisitorI {

  /**
   * Visitor method for Atoms.
   */
  public Object visitAtom(Atom atom) {
    switch (atom.getId()) {
      case sym.SYMBOL:
	return atom.getValue();
      case sym.STRING:
	return atom.getValue();
      case sym.CHAR:
	return ("#\\" + atom.getValue());
      case sym.NUMBER:
	return atom.getValue().toString();
      default:
	return atom.getValue();
    }
  }

  /**
   * Visitor method for Pairs.
   */
  public Object visitPair(Pair pair) {
    String str = "(";

    try { 
      String carstr = (String) pair.car.accept(this);
      str = str.concat(carstr);
      
      str = str.concat(" ");
    
      String cdrstr = "";
      if (!pair.cdr.isNullList()) {
	cdrstr = (String) pair.cdr.accept(this);
	if (pair.cdr.isPair()) {
	  cdrstr= cdrstr.substring(1, cdrstr.length()-1);
	}
	str = str.concat(cdrstr);
      }
      if (pair.cdr.isNullList()) {
	str = str.substring(0, str.length()-1);
      }
      
      str = str.concat(")");
      
      return str;
    } catch (SchemeJException e) {
      return "*error*";
    }
  }

  /**
   * Visitor method for the NullList.
   */
  public Object visitNullList(NullList nullList) {
    return "";
  }
}
