/* -*---------------
 * Uses: JLex 1.2.5
 * -*---------------
 * @(#) $Id$
 *
 * SchemeJ.lex: JLex specficiation for SchemeJ
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej;

import java.io.*;
import kudos.schemej.util.*;

%%
%{
  private java_cup.runtime.Symbol tok(int k, Object value) {
    if (k == sym.SYMBOL) {
      SSymbol val = new SSymbol(k, (String) value);
      java_cup.runtime.Symbol sym = 
	new java_cup.runtime.Symbol(k, -1, -1, val);
      return sym;
    }
    else if (k == sym.NUMBER) {
      SNumber val = new SNumber(k, (Double) value);
      java_cup.runtime.Symbol sym = 
	new java_cup.runtime.Symbol(k, -1, -1, val);
      return sym;
    }
    else if (k == sym.STRING) {
      SSymbol val = new SSymbol(k, (String) value);
      java_cup.runtime.Symbol sym = 
	new java_cup.runtime.Symbol(k, -1, -1, val);
      return sym;
    }
    else if (k == sym.CHAR) {
      String charstr = (String) value;
      SSymbol val = new SSymbol(k, charstr.substring(charstr.length()-1));
      java_cup.runtime.Symbol sym = 
	new java_cup.runtime.Symbol(k, -1, -1, val);
      return sym;
    }
    else if (k == sym.QUOTE) {
      SSymbol val = new SSymbol(k, (String) value);
      java_cup.runtime.Symbol sym =
	new java_cup.runtime.Symbol(k, -1, -1, val);
      return sym;
    }
    else if (k == sym.DOT) {
      SSymbol val = new SSymbol(k, (String) value);
      java_cup.runtime.Symbol sym =
	new java_cup.runtime.Symbol(k, -1, -1, val);
      return sym;
    }
    else {
      java_cup.runtime.Symbol sym = 
	new java_cup.runtime.Symbol(k, -1, -1, value);
      return sym;
    }
  }
%}

%public
%class Lexer
%implements java_cup.runtime.Scanner
%function next_token
%type java_cup.runtime.Symbol

%char
%line
%state COMMENT
%eofval{
  { return tok(sym.EOF, null); }
%eofval}


alnum=[A-Za-z0-9]
mark=[~!?@#:\$%&\*\+-\/\=\<\>\"\"\[\]]
symbol=({alnum}|{mark})+
char=#\\({alnum}|{mark})
string=\"{symbol}\"
quote=\'
dot="."
digit=[0-9]
number={digit}*("."{digit}*)?
white=[ \t\n]


%%
<YYINITIAL>"("         { return tok(sym.LPAREN, (String) "("); }
<YYINITIAL>")"         { return tok(sym.RPAREN, (String) ")"); }
<YYINITIAL>{quote}     { return tok(sym.QUOTE, (String) "quote"); }
<YYINITIAL>{char}      { return tok(sym.CHAR, yytext()); }
<YYINITIAL>{dot}       { return tok(sym.DOT, yytext()); }
<YYINITIAL>{string}    { return tok(sym.STRING, yytext()); }
<YYINITIAL>{number}    { return tok(sym.NUMBER, new Double(yytext())); }
<YYINITIAL>{symbol}    { return tok(sym.SYMBOL, yytext()); }
<YYINITIAL>{white}     {}
<YYINITIAL>";"         { yybegin(COMMENT); }
<YYINITIAL>.           {}
<COMMENT>"\n"          { yybegin(YYINITIAL); }
<COMMENT>.             {}
