/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * Converter.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej.stdlib;

import kudos.schemej.util.*;

/**
 * Converter provides methods for converting values of 
 * SchemeJ types into values of Java types.
 * This class is essential for invoking Java methods with
 * arguments of correct types.
 *
 * @author Cheoljoo Jeong
 */

public class Converter {
  /**
   * Changes the SchemeJ value, 'sValue',  into Java value
   * of type 'javaClass'.
   * Note that Method.invoke() automatically type converts
   * Integer->int, Double->double, and so on.
   * So what we have to do is convert SchemeJ *number*'s
   * which are internally represented as Double in SchemeJ
   * interpreter into values of appropriate classes such as 
   * Integer objects; we need not go as far as primitive types.
   */
  public static Object S2J(Object sValue, Class javaType) {
    // if SchemeJ value is a number
    if (sValue instanceof Double) {
      if (javaType.equals(int.class)) {
	return new Integer(((Double) sValue).intValue());
      }
      else if (javaType.equals(long.class)) {
	return new Integer(((Double) sValue).intValue());
      }
      else if (javaType.equals(float.class)) {
	return new Integer(((Double) sValue).intValue());
      }
      else if (javaType.equals(double.class)) {
	return sValue;
      }
    }
    // if SchemeJ value is either a string or a char
    else if (sValue instanceof String) {
      return sValue;
    }
    // if SchemeJ value is a symbol
    else if (sValue instanceof SSymbol) {
      return ((SSymbol) sValue).getName();
    }
    
    return sValue;
  }

  /**
   * Changes the Java values to SchemeJ values.
   */
  public static Object J2S(Object jValue) {
    if (jValue instanceof Integer) {
      return new Double(((Integer) jValue).intValue());
    }
    else {
      return jValue;
    }
  }
}

