/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * CreateObject.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej.stdlib;

import kudos.schemej.util.*;
import kudos.lang.*;
import kudos.exns.*;
import kudos.jvm.*;
import java.lang.reflect.*;

/**
 * Implements the standard library procedure 'create-object'.
 *
 * @author Cheoljoo Jeong
 */

public class CreateObject extends Procedure {
  /**
   * The "apply"
   * Visitor is not used.
   */
  public Object apply(ExprVisitorI visitor, Object[] arguments, 
		      Environment r, Continuation k) 
    throws InvalidTypeException, SchemeJException 
  {
    try {
      KudosClassLoader classLoader = KudosClassLoader.getInstance();
      Class theClass = null;
      String classname = "";

      // retrieve the 'Class' whose instance are to be created
      if (arguments[0] instanceof String) {
	classname = (String) arguments[0]; 
	theClass = classLoader.getClass(classname);
      } 
      else if (arguments[0] instanceof SSymbol) {
	classname = (String) ((SSymbol) arguments[0]).getName();
	theClass = classLoader.getClass(classname);
      }
      else if (arguments[0] instanceof Class) {
	theClass = (Class) arguments[0];
      }
      else {
	throw new InvalidTypeException("(create-object \"class-name\") " +
				       "or (create-object <class>)");
      }

      // prepare arguments, 'args', for the constructor
      Object[] args = new Object[arguments.length - 1];
      for (int i = 1; i < arguments.length; i++) {
	args[i-1] = arguments[i];
      }
      
      Object anInstance = createObject(theClass, "obj", args);
      if (anInstance == null) {
	throw new SchemeJException(SchemeJException.EvalError,
				   "object null?");
      }
      return k.resume(anInstance);
    } catch (ClassNotFoundException e) {
      return k.resume(new SchemeJException(SchemeJException.EvalError,
					   "no such class"));
    } catch (SchemeJException e) {
      return k.resume(e);
    }
    
  }

  /**
   * Creates an object of the specified class and returns it.
   */
  private Object createObject(Class theClass, String objname, Object[] args) 
    throws SchemeJException
  {
    KudosClassLoader classLoader = KudosClassLoader.getInstance();
    Constructor constructor = null;

    try {
      Class[] paramTypes = new Class[args.length];
      for (int i = 0; i < args.length; i++) {
	paramTypes[i] = args[i].getClass();
	
	// Extemporaneous type conversion.
	// We need extensive type conversion here!!!
	if (paramTypes[i].getName().equals("java.lang.Double")) {
	  paramTypes[i] = int.class;
	  args[i] = new Integer(((Double) args[i]).intValue());
	}
      }
      
      constructor = theClass.getConstructor(paramTypes);
    } catch (NoSuchMethodException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "no such class constructor");
    } catch (SecurityException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "security error while constructing");
    }
    
    try {
      Object anInstance = constructor.newInstance(args);
      classLoader.putInstance(theClass.getName(), objname, anInstance);
      return anInstance;
    } catch (InstantiationException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "instantiation error");
    } catch (IllegalAccessException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "illegal access error");
    } catch (IllegalArgumentException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "illegal argument error");
    } catch (InvocationTargetException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "invocation target error");
    } catch (Exception e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 e.getMessage());
    }

  }

  /**
   * Returns true if the given number is the same as the arity of the
   * procedure that the closure represents.
   */
  public boolean checkArity(int num) {
    if (num >= 1) {
      return true;
    }
    else {
      return false;
    }
  }
}
