/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * GetFieldValue.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej.stdlib;

import kudos.schemej.util.*;
import kudos.lang.*;
import kudos.exns.*;
import kudos.jvm.*;
import java.lang.reflect.*;

/**
 * Implements the standard library procedure 'get-field-value'.
 *
 * @author Cheoljoo Jeong
 */

public class GetFieldValue extends Procedure {
  /**
   * The "apply"
   * Visitor is not used.
   */
  public Object apply(ExprVisitorI visitor, Object[] arguments, 
		      Environment r, Continuation k) 
    throws InvalidTypeException, SchemeJException 
  {
    Object theObject = arguments[0];
    Field theField = null;
    String fieldname = "";
    try {
      if (arguments[1] instanceof String) {
	fieldname = (String) arguments[1];
      }
      else if (arguments[1] instanceof SSymbol) {
	fieldname = ((SSymbol) arguments[1]).getName();
      }
      else {
	throw new SchemeJException(SchemeJException.EvalError,
				   "(get-field-value <obj> \"field\") or " +
				   "(get-field-value <obj> 'field)");
      }
      theField = theObject.getClass().getField(fieldname);

      Object value = null;
      Class fieldty = theField.getType();
      if (fieldty.equals(boolean.class)) {
	value = new Boolean(theField.getBoolean(theObject));
      }
      else if (fieldty.equals(int.class)) {
	value = new Double(theField.getInt(theObject));
      }
      else if (fieldty.equals(long.class)) {
	value = new Double(theField.getLong(theObject));
      }
      else if (fieldty.equals(double.class)) {
	value = new Double(theField.getDouble(theObject));
      }
      else if (fieldty.equals(float.class)) {
	value = new Double(theField.getFloat(theObject));
      }
      else if (fieldty.equals(char.class)) {
	value = new Double(theField.getChar(theObject));
      }
      else {
	value = theField.get(theObject);
      }

      return k.resume(value);

    } catch (NoSuchFieldException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "no such field: " + fieldname);
    } catch (SecurityException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "cannot access field: " + fieldname);
    } catch (IllegalAccessException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "illegal field access exception: " +
				 fieldname);
    }
    
  }

  /**
   * Returns true if the given number is the same as the arity of the
   * procedure that the closure represents.
   */
  public boolean checkArity(int num) {
    if (num == 2) {
      return true;
    }
    else {
      return false;
    }
  }
}
