/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * GetMethod.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej.stdlib;

import kudos.schemej.util.*;
import kudos.lang.*;
import kudos.exns.*;
import kudos.jvm.*;
import java.lang.reflect.*;

/**
 * Implements the standard library procedure 'get-method'.
 *
 * @author Cheoljoo Jeong
 */

public class GetMethod extends Procedure {
  /**
   * The "apply"
   * Visitor is not used.
   */
  public Object apply(ExprVisitorI visitor, Object[] arguments, 
		      Environment r, Continuation k) 
    throws InvalidTypeException, SchemeJException 
  {
    KudosClassLoader classLoader = KudosClassLoader.getInstance();
    Object theObject = arguments[0];
    Method theMethod = null;
    String methodname = "";

    try {
      if (arguments[1] instanceof String) {
	methodname = (String) arguments[1];
      }
      else if (arguments[1] instanceof SSymbol) {
	methodname = ((SSymbol) arguments[1]).getName();
      }
      else {
	throw new SchemeJException(SchemeJException.EvalError,
				   "(get-method <obj> \"method\" args) or " +
				   "(get-method <obj> 'method args)");
      }

      Class[] argTypes = new Class[arguments.length - 2];
      for (int i = 2; i < arguments.length; i++) {
	argTypes[i - 2] = arguments[i].getClass();
      }
      theMethod = theObject.getClass().getMethod(methodname, argTypes);

      return k.resume(theMethod);

    } catch (NoSuchMethodException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "no such method: " + methodname);
    } catch (SecurityException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "cannot access field");
    }
    
  }

  /**
   * Returns true if the given number is the same as the arity of the
   * procedure that the closure represents.
   */
  public boolean checkArity(int num) {
    if (num == 2) {
      return true;
    }
    else {
      return false;
    }
  }
}
