/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * SendStatic.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej.stdlib;

import kudos.schemej.util.*;
import kudos.lang.*;
import kudos.exns.*;
import kudos.jvm.*;
import java.lang.reflect.*;
import java.awt.event.*;

/**
 * Implements the standard library procedure 'send-static'.
 *
 * @author Cheoljoo Jeong
 */

public class SendStatic extends Procedure {
  /**
   * The "apply"
   * Visitor is not used.
   */
  public Object apply(ExprVisitorI visitor, Object[] arguments, 
		      Environment r, Continuation k) 
    throws InvalidTypeException, SchemeJException 
  {
    KudosClassLoader classLoader = KudosClassLoader.getInstance();
    Object theObject = null;
    Class theClass = null;
    Method theMethod = null;
    String methodname = "";

    try {
      if (arguments[0] instanceof Class) {
	theClass = (Class) arguments[0];
	if ((arguments[1] instanceof String) ||
	    (arguments[1] instanceof SSymbol)) 
	{
	  if (arguments[1] instanceof String) {
	    methodname = (String) arguments[1]; 
	  } 
	  else {
	    methodname = (String) ((SSymbol) arguments[1]).getName();
	  }
	  
	  Class[] argTypes = new Class[arguments.length - 2];
	  for (int i = 2; i < arguments.length; i++) {
	    Class cls = arguments[i].getClass();
	    if (arguments[i] instanceof Boolean) {
	      cls = boolean.class;
	    }
	    else if (arguments[i] instanceof Double) {
	      cls = int.class;
	    }
	    argTypes[i-2] = cls;
	  }

	  if (argTypes.length == 0) {
	    theMethod = theClass.getMethod(methodname, null);
	  }
	  else {
	    theMethod = theClass.getMethod(methodname, argTypes);
	  }
	}
	else if (arguments[1] instanceof Method) {
	  theMethod = (Method) arguments[1];
	}
	if (theMethod == null) {
	  throw new InvalidTypeException("no such method: " + methodname);
	}
      }
      else {
	throw new InvalidTypeException("(send-static <class> <method> " +
				       "<arg_1> <arg_2> ...): specify method");
      }

      Object[] args = new Object[arguments.length - 2];

      for (int i = 2; i < arguments.length; i++) {
	if (arguments[i] instanceof Double) {
	  args[i-2] = new Integer(((Double) arguments[i]).intValue());
	}
	else {
	  args[i-2] = arguments[i];
	}
      }
      
      Object value = Converter.J2S(theMethod.invoke(arguments[0], args));

      return k.resume(value);

    } catch (NoSuchMethodException e) {
      try {
	Method[] methods = theObject.getClass().getMethods();
	for (int i = 0; i < methods.length; i++) {
	  if (methods[i].getName().equals(methodname)) {
	    theMethod = methods[i];
	    Object[] args = new Object[arguments.length - 2];
	    
	    for (int j = 2; j < arguments.length; j++) {
	      if (arguments[j] instanceof Double) {
		args[j - 2] = new Integer(((Double) arguments[j]).intValue());
	      }
	      else {
		args[j - 2] = arguments[j];
	      }
	    }

	    return 
	      k.resume(Converter.J2S(theMethod.invoke(arguments[0], args)));
	  }
	}
      } catch (IllegalAccessException e2) {
	throw new SchemeJException(SchemeJException.EvalError,
				   "illegal access");
      } catch (IllegalArgumentException e2) {
	throw new SchemeJException(SchemeJException.EvalError,
				   "illegal argument");
      } catch (InvocationTargetException e2) {
	throw new SchemeJException(SchemeJException.EvalError,
				   "invocation target error");
      }
      throw new SchemeJException(SchemeJException.EvalError,
				 "no such method");
    } catch (IllegalAccessException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "illegal access");
    } catch (IllegalArgumentException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "illegal argument");
    } catch (InvocationTargetException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "invocation target error");
    }
  }

  /**
   * Returns true if the given number is the same as the arity of the
   * procedure that the closure represents.
   */
  public boolean checkArity(int num) {
    if (num >= 2) {
      return true;
    }
    else {
      return false;
    }
  }
}
