/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * SetValue.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej.stdlib;

import kudos.schemej.util.*;
import kudos.lang.*;
import kudos.exns.*;
import kudos.jvm.*;
import java.lang.reflect.*;

/**
 * Implements the standard library procedure 'set-field-value'.
 * Form: (set-value <field> <value>)
 *
 * @author Cheoljoo Jeong
 */

public class SetFieldValue extends Procedure {
  /**
   * The "apply"
   * Visitor is not used.
   */
  public Object apply(ExprVisitorI visitor, Object[] arguments, 
		      Environment r, Continuation k) 
    throws InvalidTypeException, SchemeJException 
  {
    try {
      Object theObject = arguments[0];
      Field theField = null;
      String fieldname = "";
      if (arguments[1] instanceof String) {
	fieldname = (String) arguments[1];
      }
      else if (arguments[1] instanceof SSymbol) {
	fieldname = ((SSymbol) arguments[1]).getName();
      }
      else {
	throw new SchemeJException(SchemeJException.EvalError,
				   "(set-field-value <o> 'field <val>)");
      }
      theField = theObject.getClass().getField(fieldname);

      Object value = arguments[2];
      Class fieldty = theField.getType();
      if (fieldty.equals(boolean.class)) {
	if (value instanceof Boolean) {
	  theField.setBoolean(theObject, ((Boolean) value).booleanValue());
	}
	else {
	  throw new SchemeJException("field type mismatch");
	}
      }
      else if (fieldty.equals(int.class)) {
	if (value instanceof Double) {
	  theField.setInt(theObject, ((Double) value).intValue());
	}
	else {
	  throw new SchemeJException("field type mismatch");
	}
      }
      else if (fieldty.equals(short.class)) {
	if (value instanceof Double) {
	  theField.setShort(theObject, ((Double) value).shortValue());
	}
	else {
	  throw new SchemeJException("field type mismatch");
	}
      }
      else if (fieldty.equals(long.class)) {
	if (value instanceof Double) {
	  theField.setDouble(theObject, ((Double) value).longValue());
	}
	else {
	  throw new SchemeJException("field type mismatch");
	}
      }
      else if (fieldty.equals(double.class)) {
	if (value instanceof Double) {
	  theField.setDouble(theObject, ((Double) value).doubleValue());
	}
	else {
	  throw new SchemeJException("field type mismatch");
	}
      }
      else if (fieldty.equals(float.class)) {
	if (value instanceof Float) {
	  theField.setFloat(theObject, ((Float) value).floatValue());
	}
	else {
	  throw new SchemeJException("field type mismatch");
	}
      }
      else if (fieldty.equals(char.class)) {
	if (value instanceof String) {
	  if (((String) value).length() == 1) {
	    theField.setChar(theObject, ((String) value).charAt(0));
	  }
	}
	else {
	  throw new SchemeJException("field type mismatch");
	}
      }
      else {
	theField.set(theObject, value);
      }
      
      return k.resume(value);
      
    } catch (NoSuchFieldException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "no such field ");
    } catch (SecurityException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "cannot access field");
    } catch (IllegalAccessException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "illegal field access exception");
    } catch (NullPointerException e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "the object is null");
    } catch (ExceptionInInitializerError e) {
      throw new SchemeJException(SchemeJException.EvalError,
				 "initializer failure");
    }
    
  }

  /**
   * Returns true if the given number is the same as the arity of the
   * procedure that the closure represents.
   */
  public boolean checkArity(int num) {
    if (num == 3) {
      return true;
    }
    else {
      return false;
    }
  }
}

