/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * Atom.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej.util;

import kudos.exns.*;


/**
 * Implements the atoms of Lisp/Scheme.
 *
 * @author Cheoljoo Jeong
 */
public class Atom extends SExpr {
  protected int id;
  protected Object value;

  /**
   * constructor
   */
  public Atom(int id, Object value) {
    this.id = id;
    try {
      setValue(value);
    } catch (InvalidTypeException e) {
      e.printStackTrace();
    }
  }

  
  /**
   * Checks the type of value and sets it.
   */
  public void setValue(Object value) throws InvalidTypeException {
    if (id == Token.Symbol) {
      if (value instanceof String) {
	this.value = value;
      }
      else {
	throw new InvalidTypeException();
      }
    }
    else if (id == Token.Number) {
      if (value instanceof Double) {
	this.value = value;
      }
      else {
	throw new InvalidTypeException();
      }
    }
    else {
      this.value = value;
    }
  }

  /**
   * Returns true iff this instance is atom;
   */
  public boolean isAtom() {
    return true;
  }

  /**
   * Returns true iff this instance is pair;
   */
  public boolean isPair() {
    return false;
  }

  /**
   * Returns true iff this instnace is null;
   */
  public boolean isNullList() {
    return false;
  }

  /**
   * Returns the length of this list.
   */
  public int length() {
    return 1;
  }

  /**
   * Returns the token id of this atom.
   */
  public int getId() {
    return id;
  }

  /**
   * Returns the name of this atom.
   */
  public Object getValue() {
    return value;
  }

  /**
   * Acceptor method for visitor pattern.
   */
  public Object accept(SExprVisitorI visitor) throws SchemeJException {
    try {
      return visitor.visitAtom(this);
    } catch (SchemeJException e) {
      throw e;
    }
  }

}




