/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * Pair.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej.util;

import kudos.exns.*;

/**
 * Implements the dotted pair structure of List/Scheme
 *
 * @author Cheoljoo Jeong
 */
public class Pair extends SExpr {
  public SExpr car;
  public SExpr cdr;

  /**
   * constructor
   */
  public Pair(SExpr car, SExpr cdr) {
    this.car = car;
    this.cdr = cdr;
  }


  /**
   * Returns true iff this instance is atom;
   */
  public boolean isAtom() {
    return false;
  }
  
  /**
   * Returns true iff this instance is pair;
   */
  public boolean isPair() {
    return true;
  }

  /**
   * Returns true iff this instnace is null;
   */
  public boolean isNullList() {
    return false;
  }

  /**
   * Returns the length of this list.
   */
  public int length() {
    if (cdr == null) {
      return 1;
    }
    else {
      return cdr.length() + 1;
    }
  }

  /**
   * compares this pair with another pair for equality;
   * overrides equals(Object obj) of java.lang.Object
   */
  public boolean equals(Pair pair) {
    if (this == pair) {
      return true;
    }
    if (pair == null) {
      return false;
    }
    
    if (car.equals(pair.car)) {
      if (cdr.equals(pair.cdr)) {
	return true;
      }
      return false;
    }
    else {
      return false;
    }
  }

  /**
   * static method for comparing two pairs for equality
   */
  public static boolean equals(Pair pair1, Pair pair2) {
    if (pair1 != null) {
      return pair1.equals(pair2);
    }
    else if (pair2 == null) {
      return true;
    }
    else {
      return false;
    }
  }

  /**
   * Acceptor method for visitor pattern.
   */
  public Object accept(SExprVisitorI visitor) throws SchemeJException {
    try {
      return visitor.visitPair(this);
    } catch (SchemeJException e) {
      throw e;
    }
  }

}




