/* -*---------------------------------------------------------
 * Uses: Classic VM (build JDK-1.2-V, green threads, sunwjit)
 * -*---------------------------------------------------------
 * @(#) $Id$
 *
 * SList.java: 
 *   - created by Cheoljoo Jeong
 *
 * Copyleft(c) 2000. Cheoljoo Jeong. All lefts reserved.
 *
 * $Log$
 */

package kudos.schemej.util;

import java.util.*;


/**
 * Implements the list structure of List/Scheme
 *
 * @author Cheoljoo Jeong
 */
public class SList extends LinkedList {
  /**
   * constructor
   */
  public SList() {
    super();
  }

  /**
   * constructor
   */
  public SList(SExpr p) {
    SList list = makeSListFromPair(p);
    ListIterator iterator = list.listIterator(0);
    while (iterator.hasNext()) {
      add(iterator.next());
    }
  }

  /**
   * Make a SList from Pair.
   */
  private SList makeSListFromPair(SExpr p) {
    // if p is a non-null pair
    if (p instanceof Pair && p != null) {
      SList l = new SList();

      // if car is atom
      if (((Pair) p).car instanceof Atom) {
	l.add(((Pair) p).car);
      }
      // else if car is pair
      else if (((Pair) p).car instanceof Pair) {
	l.add(makeSListFromPair(((Pair) p).car));
      }
      
      // make SList from cdr recursively
      if (((Pair) p).cdr != null) {
	l.append(makeSListFromPair(((Pair) p).cdr));
      }
      
      return l;
    } 
    else
      return null;
  }

  /**
   * Append the elements of the second list to the end of the 
   * this list.
   */
  public void append(SList l2) {
    ListIterator iterator = l2.listIterator(0);
    while (iterator.hasNext()) {
      add(iterator.next());
    }
  }

  /**
   * Returns the car of this list.
   */
  public Object car() {
    return first();
  }

  /**
   * Returns the cdr of this list.
   */
  public SList cdr() {
    if (this.size() <= 1) {
      return null;
    }
    else {
      SList list = new SList();
      ListIterator iterator = this.listIterator(0);
      iterator.next();
      while (iterator.hasNext()) {
	list.add(iterator.next());
      }
      return list;
    }
  }

  /**
   * Returns the first element of the list.
   */
  public Object first() {
    if (!isEmpty()) {
      return getFirst();
    }
    else
      return null;
  }

  /**
   * Returns the second element of the list.
   */
  public Object second() {
    if (this.size() >= 2) {
      return get(1);
    }
    else 
      return null;
  }

  /**
   * Returns the third element of the list.
   */
  public Object third() {
    if (this.size() >= 3) {
      return get(2);
    }
    else
      return null;
  }

  /**
   * Returns the fourth element of the list.
   */
  public Object fourth() {
    if (this.size() >= 4) {
      return get(3);
    }
    else
      return null;
  }
  
  /**
   * Cons: in fact, this implementation is wrong.
   * Lacking consideration for dotted pair!
   */
  public SList cons(Object obj) {
    if (obj instanceof SList) {
      if (obj != null) {
	ListIterator it = ((SList) obj).listIterator(0);
	while (it.hasNext()) {
	  Object o = it.next();
	  this.addLast(o);
	}
      }
    }
    else {
      this.addLast(obj);
    }
    return this;
  }

  /**
   * Pretty print.
   */
  public static void prettyPrint(SList list) {
    localPrettyPrint(list);
    System.out.println("");
  }
  
  /**
   * Pretty print for SList's.
   */
  private static void localPrettyPrint(SList list) {
    if (list instanceof SList) {
      System.out.print("(");

      ListIterator it = list.listIterator(0);
      while (it.hasNext()) {
	Object o = it.next();
	if (o instanceof SList) {
	  localPrettyPrint((SList) o);
	}
	else if (o instanceof Atom) {
	  localPrettyPrint((Atom) o);
	}
	if (it.hasNext()) {
	  System.out.print(" ");
	}
      }
      System.out.print(")");
    }
  }

  /**
   * Pretty print for Atom's.
   */
  private static void localPrettyPrint(Atom atom) {
      System.out.print(atom.getValue());
  }

  /**
   * Returns true iff the two given SList's are same in the sence
   * of "Scheme eqv? procedure."
   */
  public boolean equals(SList list) {
    if (list instanceof SList) {
      if (list.size() == this.size()) {
	ListIterator it1 = this.listIterator(0);
	ListIterator it2 = list.listIterator(0);
	while (it1.hasNext()) {
	  Object o1 = it1.next();
	  Object o2 = it2.next();
	  if (!o1.equals(o2)) {
	    return false;
	  }
	}
	return true;
      }
      else {
	return false;
      }
    }
    else {
      return false;
    }
  }
}




